﻿;################################################
;# 
;# Program : Playlister
;#
;# Version: v1.0
;# Date: XX/10/2016
;# Author: Paul Vince
;# Target Compiler: Purebasic 5.5+
;# Target OS: Windows
;#
;# Code Credits
;# ------------
;# Tagreader by Thomas (ts-soft) Schulz
;# mp3length procedure by case
;# Capitalizestring Procedure by Luis/Little john
;# DragNDrop by netmaestro
;# Recursive directory search by firace & AND51
;#
;################################################
;
; Changes
; -------
;
; tagreader
; 1. Corrected frame size calculation on TXXX,COMM and APIC frames
; 2. fixed a bug that would prevent the last tag in the map being shown
; 3. Swapped to case statement for tag lookup
;
; mp3length procedure
; 1. Added ID3 v2.x header skip
; 2. Changed header generation to string in memory
; 3. Mp3 file parse uses memory compare instead of Mid function (A lot faster)
; 4. Commented out unused checks
; 5. Added VBR/CBR bitrate calculations
; 6. Fixed uneven filesize error
; 7. Fixed corrupt MP3 crash
;
; capitalizestring procedure
; 1.added check for first letter after left bracket "(" and minus sign "-"
;
; v0.1
; Combined tagreader and mp3length procedures
; Added Square bracket to capitalize string

DeclareModule TagReader
  Interface iTagReader
    Destroy()
    GetTagList.s()
    GetContent.s(Tag.s)
  EndInterface
  
  Declare New_MP3TagObj(FileName.s)
EndDeclareModule

Module TagReader
  EnableExplicit
  
  Structure TagReader
    ID.a[3]
    version.a[2]
    flags.b
    size.l
    *memory
  EndStructure
  
  Structure TagReader_Class
    *vTable
    Map Tags.s()
    AvailableTags.s
  EndStructure
  
Procedure Destroy(*obj.TagReader_Class)
    If *obj
      FreeMap(*obj\Tags())
      FreeMemory(*obj)
    EndIf
EndProcedure
  
Procedure.s GetTagList(*obj.TagReader_Class)
    If *obj
      ProcedureReturn *obj\AvailableTags
    EndIf
EndProcedure
  
Procedure.s GetContent(*obj.TagReader_Class, Tag.s)
  If *obj
    ProcedureReturn *obj\Tags(Tag)
  EndIf
EndProcedure
  
Procedure New_MP3TagObj(FileName.s)
  
  Dim genre.s(147)
  
  Protected i, size, TextEncoding.b, ID3.TagReader
  Protected frame, FrameID.s, content.s, b
  Protected b1,b2,b3,b4,firstsync,v,l,crc,bitrate,freq,padded,framesize,total,hour,min,sec,fsize,bit,fnum,corrupt
  Protected totalsec.f
  Protected head.s
  Protected bitout.s
  Protected *mem,*file
  Global FF,pos
  Global *obj.TagReader_Class   
  If FileSize(FileName) <= 0 : ProcedureReturn #False : EndIf
  
  *obj = AllocateMemory(SizeOf(TagReader_Class))
  
  If *obj
    InitializeStructure(*obj, TagReader_Class)
    *obj\vTable = ?vTable
    
    Restore genre
    For i = 0 To 146
      Read.s genre(i)
    Next
    
    FF = ReadFile(#PB_Any, FileName,#PB_File_SharedRead)
    
    *file=AllocateMemory(Lof(ff))
    
    If FF
      ; ----- ID3v1 -----
      ReadData(FF, *file, Lof(FF))
      CloseFile(FF)
      pos=MemorySize(*file)
      If PeekS(*file+(pos-128), 3, #PB_Ascii) = "TAG"
        content = Trim(PeekS(*file+(pos-125), 30, #PB_Ascii))
        If content <> ""
          *obj\Tags("TITLE") = content
          *obj\AvailableTags + "TITLE" + #LF$
        EndIf
        content = Trim(PeekS(*file+(pos-95), 30, #PB_Ascii))        
        If content <> ""
          *obj\Tags("ARTIST") = content
          *obj\AvailableTags + "ARTIST" + #LF$
        EndIf
        content = Trim(PeekS(*file+(pos-65), 30, #PB_Ascii))
        If content <> ""
          *obj\Tags("ALBUM") = content
          *obj\AvailableTags + "ALBUM" + #LF$
        EndIf
        content = Trim(PeekS(*file+(pos-35), 4, #PB_Ascii))
        If content <> ""
          *obj\Tags("YEAR") = content
          *obj\AvailableTags + "YEAR" + #LF$
        EndIf
        content = Trim(PeekS(*file+(pos-31), 30, #PB_Ascii))
        If content <> ""
          *obj\Tags("COMMENT") = content
          *obj\AvailableTags + "COMMENT" + #LF$
        EndIf
        b = PeekA(*file+(pos-2))
        If b > 0
          *obj\Tags("TRACK") = Str(b)
          *obj\AvailableTags + "TRACK" + #LF$
        EndIf
        b = PeekA(*file+(pos-1))
        If b > -1 And b < 147
          *obj\Tags("GENRE") = genre(b)
          *obj\AvailableTags + "GENRE" + #LF$
        EndIf
      EndIf 
      
      ; ----- ID3v2.x -----
      pos=0
      If PeekS(*file, 3, #PB_Ascii) <> "ID3" ; { kein ID3v2 vorhanden
        If *obj\AvailableTags = ""
          FreeMemory(*obj)
          ProcedureReturn #False
        Else
          ProcedureReturn *obj
        EndIf
      EndIf 
      pos+10 ; Move past ID3 headet
      For i = 3 To 0 Step -1
        ID3\size + Asc(PeekS(*file+(pos-i),1,#PB_Ascii)) << (7 * i)
      Next
      ; --- FrameHeader (10 Byte) ---
      Repeat
        size = 0
        frame = PeekL(*file + pos)
        If frame
          ; FrameID (4 Byte)
          FrameID = PeekS(*file+pos, 4, #PB_Ascii)
          pos + 4
          If FrameID <> ""
            ; FrameSize (4 Byte)              
            For i = 0 To 3
              size= (size << 8) | PeekA(*file + pos + i)
            Next i   
            ; Move past frame header 4 bytes              
            pos + 4              
            ; Flags (2 Byte)
            pos + 2
            ; TextEncoding (1 Byte)
            TextEncoding = PeekB(*file + pos)&$FF
            pos + 1
            If TextEncoding = 0
              content = PeekS(*file + pos, size-1, #PB_Ascii)
            ElseIf TextEncoding = 1
              content = PeekS(*file + pos + 2, (size / 2) -1, #PB_Unicode)
            EndIf
            pos + size - 1
            If content <> ""
              If Left(content, 1) <> Chr(255)
                *obj\Tags(FrameID) = content
                *obj\AvailableTags + FrameID + #LF$
              EndIf
            EndIf
          EndIf
        Else
          Break
        EndIf
      Until pos >= ID3\size
      
      bit=0
      fnum=0
      
      corrupt=0
      
      *mem=AllocateMemory(33)
      
      Repeat    ;Begin Loop for Frame headers
        retry:
        head.s   
        
        b1=PeekB(*file+pos)   ; Read in the 4 byte header
        b2=PeekB(*file+pos+1) ;
        b3=PeekB(*file+pos+2) ;
        b4=PeekB(*file+pos+3) ;
        
        pos+4
        
        PokeS(*mem,RSet(Bin(b1&$ff),8,"0"),8,#PB_Ascii) ; convert into 32 bit binary string in memory
        PokeS(*mem+8,RSet(Bin(b2&$ff),8,"0"),8,#PB_Ascii)
        PokeS(*mem+16,RSet(Bin(b3&$ff),8,"0"),8,#PB_Ascii)
        PokeS(*mem+24,RSet(Bin(b4&$ff),8,"0"),8,#PB_Ascii)
        
        If PeekS(*mem,11,#PB_Ascii)="11111111111" ; Frame header check
          firstsync=1       
          ; MPEG Version Check
          Select PeekS(*mem+11,2,#PB_Ascii)
            Case "00"
              v=25  ; "mpeg 2.5"
            Case "01"    
              ; "reserved"
            Case "10"
              v=2   ; "mpeg 2"
            Case "11"
              v=1   ; "mpeg 1"
          EndSelect
          ; Layer Check
          Select PeekS(*mem+13,2,#PB_Ascii)
            Case "00" 
              ; "reserved"
            Case "01"
              l=3 ; layer 3"
            Case "10"
              l=2 ; layer 2"
            Case "11"
              l=1 ; "layer 1"
          EndSelect
          ; CRC Protection Check
          Select PeekS(*mem+15,1,#PB_Ascii)
            Case "0"
              crc=0; "protected by crc"
            Case "1"
              crc=1; "no crc protected"
          EndSelect
          ; Determine Frame Bitrate
          bitrate=0
          If  l=1   ; LAYER 1
            Select PeekS(*mem+16,4,#PB_Ascii)
              Case "0000"
                ; "free"
              Case "0001"
                bitrate=32
              Case "0010"
                bitrate=64
              Case "0011"
                bitrate=96
              Case "0100"
                bitrate=128
              Case "0101"
                bitrate=160
              Case "0110"
                bitrate=192
              Case "0111"
                bitrate=224
              Case "1000"
                bitrate=256
              Case "1001"
                bitrate=288
              Case "1010"
                bitrate=320
              Case "1011"
                bitrate=352
              Case "1100"
                bitrate=384
              Case "1101"
                bitrate=416
              Case "1110"
                bitrate=448
              Case "1111"
                ; "bad"
            EndSelect
          EndIf
          If  l=2 ; LAYER 2
            Select PeekS(*mem+16,4,#PB_Ascii)
              Case "0000"
                ; "free"
              Case "0001"
                bitrate=32
              Case "0010"
                bitrate=48
              Case "0011"
                bitrate=56
              Case "0100"
                bitrate=64
              Case "0101"
                bitrate=80
              Case "0110"
                bitrate=96
              Case "0111"
                bitrate=112
              Case "1000"
                bitrate=128
              Case "1001"
                bitrate=160
              Case "1010"
                bitrate=192
              Case "1011"
                bitrate=224
              Case "1100"
                bitrate=256
              Case "1101"
                bitrate=320
              Case "1110"
                bitrate=384
              Case "1111"
                ; "bad"
            EndSelect
          EndIf
          If  l=3 ;LAYER 3
            Select PeekS(*mem+16,4,#PB_Ascii)
              Case "0000"
                ; "free"
              Case "0001"
                bitrate=32
              Case "0010"
                bitrate=40
              Case "0011"
                bitrate=48
              Case "0100"
                bitrate=56
              Case "0101"
                bitrate=64
              Case "0110"
                bitrate=80
              Case "0111"
                bitrate=96
              Case "1000"
                bitrate=112
              Case "1001"
                bitrate=128
              Case "1010"
                bitrate=160
              Case "1011"
                bitrate=192
              Case "1100"
                bitrate=224
              Case "1101"
                bitrate=256
              Case "1110"
                bitrate=320
              Case "1111"
                ; "bad"
            EndSelect
          EndIf
          ; Sampling Rate Frequency index
          freq=0
          If v=1
            Select PeekS(*mem+20,2,#PB_Ascii)
              Case "00"
                freq=44100
              Case "01"
                freq=48000
              Case "10"
                freq=32000
              Case "11"
                ;"Reserved"
            EndSelect
          EndIf
          If v=2
            Select PeekS(*mem+20,2,#PB_Ascii)
              Case "00"
                freq=22050
              Case "01"
                freq=24000
              Case "10"
                freq=16000
              Case "11"
                ;"Reserved"
            EndSelect
          EndIf
          If v=25
            Select PeekS(*mem+20,2,#PB_Ascii)
              Case "00"
                freq=11025
              Case "01"
                freq=12000
              Case "10"
                freq=8000
              Case "11"
                ;"Reserved"
            EndSelect
          EndIf         
          ; Check for Frame Padding
          Select PeekS(*mem+22,1,#PB_Ascii)
            Case "0"
              padded=0
            Case "1"
              padded=1
          EndSelect
          ; Channel Mode
          ;         Select PeekS(*mem+23,2,#PB_Ascii)
          ;           Case "00"
          ;             ; "stereo"
          ;           Case "01"
          ;             ; "joint stereo"
          ;           Case "10"
          ;             ; "dual channel"
          ;           Case "11"
          ;             ; "mono"
          ;         EndSelect
          ;         ; Mode Extension (Joint Stereo)
          ;         Select PeekS(*mem+25,2,#PB_Ascii)
          ;           Case "00"
          ;             ; "intensity stereo off: ms stereo off"
          ;           Case "01"
          ;             ; "intensity stereo off: ms stereo off"
          ;           Case "10"
          ;             ; "intensity stereo off: ms stereo on"
          ;           Case "11"
          ;             ; "intensity stereo off: ms stereo on"
          ;         EndSelect
          ;         ; Copyright Check
          ;         Select PeekS(*mem+27,1,#PB_Ascii)
          ;           Case "0"
          ;             ; "audio not copyrighted"
          ;           Case "1"
          ;             ; "audio copyrighted"   
          ;         EndSelect  
          ;         ;Original Check
          ;         Select PeekS(*mem+28,1,#PB_Ascii)
          ;           Case "0"
          ;             ; "copy of original media"
          ;           Case "1"
          ;             ; "original media"   
          ;         EndSelect
          ;         ; Emphasis Indication
          ;         Select PeekS(*mem+29,2,#PB_Ascii)
          ;           Case "00"
          ;             ; "none"
          ;           Case "01"
          ;             ; "50/15 ms"   
          ;           Case "10"
          ;             ; "reserved"
          ;           Case "11"
          ;             ; "ccit j.17"
          ;         EndSelect
        Else
          If firstsync=1
            Break
          Else
            pos-3
            Goto retry
          EndIf
        EndIf
        If freq=0
          corrupt=1
          Break
        Else
          framesize=((144*(bitrate*1000))/freq)+padded      ; Framesize Calculation
        EndIf
        totalsec.f=totalsec+(((framesize*8)/1000)/bitrate) ; Total Seconds Calculation
        i=pos+(framesize-4)                                ; Bytes to next frame
        If i<=MemorySize(*file) : pos=i : EndIf            ; Move to next frame as long as it's less than the total file size
        fnum+1                                             ; Frame Count
        bit+bitrate                                        ; Bitrate Count
        If bit/fnum<>bitrate : bitout="VBR" : Else : bitout="CBR" : EndIf   ; Constant / Variable Bitrate Check
      ForEver
      If corrupt=0
        total=Int(totalsec)                                  ; Time Conversion
        hour=(total/3600)
        total=(total-(hour*3600))
        min=Int(total/60)
        sec=total-(min*60)
        
        *obj\Tags("LENGTH") = RSet(Str(hour),2,"0")+":"+RSet(Str(min),2,"0")+":"+RSet(Str(sec),2,"0")
        *obj\AvailableTags + "LENGTH" + #LF$
        *obj\Tags("BITRATE") = Str(bit/fnum)
        *obj\AvailableTags + "BITRATE" + #LF$
        *obj\Tags("BITTYPE") = bitout
        *obj\AvailableTags + "BITTYPE" + #LF$
      Else
        *obj\Tags("LENGTH") = "ERROR"
        *obj\AvailableTags + "LENGTH" + #LF$
        *obj\Tags("BITRATE") = "0"
        *obj\AvailableTags + "BITRATE" + #LF$
        *obj\Tags("BITTYPE") = "?"
        *obj\AvailableTags + "BITTYPE" + #LF$
      EndIf
      
      *obj\AvailableTags = Left(*obj\AvailableTags, Len(*obj\AvailableTags) - 1)
      
      FreeMemory(*file)        
      
      ProcedureReturn *obj
      
    EndIf
    
  EndIf
  
  ProcedureReturn #False
  
EndProcedure 

DataSection
  vTable:
  Data.i @Destroy()
  Data.i @GetTagList()
  Data.i @GetContent()
  genre:
  Data.s "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", "Grunge", "Hip-Hop", "Jazz", "Metal"
  Data.s "New Age", "Oldies", "Other", "Pop", "R&B", "Rap", "Reggae", "Rock", "Techno", "Industrial"
  Data.s "Alternative", "Ska", "Death Metal", "Pranks", "Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk"
  Data.s "Fusion", "Trance", "Classical", "Instrumental", "Acid", "House", "Game", "Sound Clip", "Gospel", "Noise"
  Data.s "AlternRock", "Bass", "Soul", "Punk", "Space", "Meditative", "Instrumental Pop", "Instrumental Rock", "Ethnic", "Gothic"
  Data.s "Darkwave", "Techno-Industrial", "Electronic", "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy", "Cult", "Gangsta"
  Data.s "Top 40", "Christian Rap", "Pop/Funk", "Jungle", "Native American", "Cabaret", "New Wave", "Psychadelic", "Rave", "Showtunes"
  Data.s "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz", "Polka", "Retro", "Musical", "Rock & Roll", "Hard Rock"
  Data.s "Folk", "Folk-Rock", "National Folk", "Swing", "Fast Fusion", "Bebob", "Latin", "Revival", "Celtic", "Bluegrass"
  Data.s "Avantgarde", "Gothic Rock", "Progressive Rock", "Psychedelic Rock", "Symphonic Rock", "Slow Rock", "Big Band", "Chorus", "Easy Listening", "Acoustic"
  Data.s "Humour", "Speech", "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony", "Booty Bass", "Primus", "Porn Groove"
  Data.s "Satire", "Slow Jam", "Club", "Tango", "Samba", "Folklore", "Ballad", "Power Ballad", "Rhythmic Soul", "Freestyle"
  Data.s "Duet", "Punk Rock", "Drum Solo", "Acapella", "Euro-House", "Dance Hall", "Goa", "Drum & Bass", "Club-House", "Hardcore", "Terror", "indie", "Brit Pop", "Negerpunk"
  Data.s "Polsk Punk", "Beat", "Christian Gangsta Rap", "Heavy Metal", "Black Metal", "Crossover", "Comteporary Christian"
  Data.s "Christian Rock", "Merengue", "Salsa", "Trash Metal", "Anime", "JPop", "Synth Pop"
EndDataSection

EndModule

EnableExplicit

Enumeration
  
  #Main_Window
  #Main_Menu
  #Main_Popup_Menu
  #Main_Status
  
  #Track_List
  
  #Edit_Window
  #Edit_Frame
  #Edit_Path
  #Edit_Artist
  #Edit_Title
  #Edit_Time
  #Edit_Bitrate
  #Edit_BitType
  #Edit_BPM
  #Edit_Close
  #Edit_Cancel
  
  #Split_Window
  #Split_Save
  #Split_List
  #Split_Time
  #Split_Files
  #Split_Spaces
  #Split_Separator
  #Split_Numbering
  #Split_Example
  #Split_Name
  #Split_Split_Num
  #Split_Split_Len
  #Split_Panel
  #Split_Preview_Editor
  
  #Tool_Container
  #Tool_OpenM3U
  #Tool_OpenFolder
  #Tool_SaveM3U
  #Tool_SaveSplit
  #Tool_Cut
  #Tool_Copy
  #Tool_Paste
  #Tool_Edit
  #Tool_Find
  #Tool_AutoFind
  #Tool_Play
  #Tool_Pause
  #Tool_Stop
  #Tool_Info
  #Tool_Volume
  #Tool_Target
  #Tool_Seek
  #Tool_Clear
  #Tool_OpenM3U_img
  #Tool_OpenFolder_img
  #Tool_SaveM3U_img
  #Tool_SaveSplit_img
  #Tool_Cut_img
  #Tool_Copy_img
  #Tool_Paste_img
  #Tool_Edit_img
  #Tool_Find_img
  #Tool_AutoFind_img
  #Tool_Play_img
  #Tool_Pause_img
  #Tool_Stop_img
  #Tool_Clear_img
  #Tool_Target_img
  
  #M3U_File
  #MP3_file
  #MP3_Play_File
  
EndEnumeration

Global prog_title.s="Playlister V1.0"
Global.l event,gadget,type,oldgadgetlist,oldoldgadgetlist
Global file.s
Global m3ufile.s
Global m3u_text.s
Global output.s
Global cut.s
Global path.s
Global folder.s
Global count,tcount,tc
Global artist.s, title.s
Global capital.b=#True
Global times.i=1
Global sort.b=#True
Global overwrite.b=#True
Global mp3load.b=#False
Global pause.b=#False
Global lastsort=0
Global found.i
Global result.i
Global size.i
Global fnum.l
Global DragItem,TargetItem,UndoLoc

Global M_Volume, M_Seek
Global ratio.f
Global Split_Filename
Global s_separator=1
Global s_numbering=1
Global s_time=1
Global s_spaces=0
Global s_filenum=10
Global s_file.s=""
Global s_type=0

Global.s MP3, tList, playingtrack
Global.i i, c, oldpos=0, newpos
Global.TagReader::iTagReader TR
Global.f f , j, mp3pos, mp3ratio, fps
Global *Relative

Structure Track_Data
  t_path.s
  t_artist.s
  t_title.s
  t_time.s
  t_bitrate.s
  t_vbr.s
  t_bpm.s
EndStructure

Structure Backup_Data
  b_path.s
  b_artist.s
  b_title.s
  b_time.s
  b_bitrate.s
  b_vbr.s
  b_bpm.s
EndStructure

Structure Undo_Data
  u_path.s
  u_artist.s
  u_title.s
  u_time.s
  u_bitrate.s
  u_vbr.s
  u_bpm.s
EndStructure

Structure Split_Data
  sp_name.s
  List sp_file.s()
  sp_length.s
EndStructure

Global NewList tracks.Track_Data()

Global NewList backup.Backup_Data()

Global NewList undo.Undo_Data()

Global NewList split.Split_Data()

Procedure.s CapitalizeString(str$)
  Protected *p.Character = @str$
  Protected newWord.i = #True
  
  While *p\c
    If newWord
      *p\c = Asc(UCase(Chr(*p\c)))
      newWord = #False
    Else
      *p\c = Asc(LCase(Chr(*p\c)))
    EndIf
    If *p\c = ' '
      newWord = #True
    EndIf
    If *p\c = '('
      newWord = #True
    EndIf 
    If *p\c = '['
      newWord = #True
    EndIf 
    If *p\c = '-'
      newWord = #True
    EndIf 
    *p + SizeOf(Character)
  Wend
  
  ProcedureReturn Trim(str$)
EndProcedure

Procedure Remove_Dupes()
  
  Protected NewMap D_Temp.s()
  Protected NewList Temp.i()
  
  ForEach tracks()
    D_Temp(tracks()\t_path)=Str(ListIndex(tracks()))
  Next
  
  Count=0  
  
  ForEach tracks()
    If FindMapElement(D_Temp(),tracks()\t_path)
      If Val(D_Temp(tracks()\t_path)) <> ListIndex(tracks())
        AddElement(Temp())
        Temp()=ListIndex(tracks())
      EndIf
    EndIf
  Next
  
  ForEach Temp()
    SelectElement(tracks(),temp()-count)
    DeleteElement(tracks(),1)
    count+1
  Next
  
  FreeMap(D_Temp())
  FreeList(Temp())
  
EndProcedure

Procedure.i DragCallBack(Action)
  Protected pt.POINT, rc.RECT
  Protected top
  GetCursorPos_(pt)
  GetWindowRect_(GadgetID(#Track_List), rc)
  If PtInRect_(rc, pt\x + pt\y<<32)
    ;Check if the underlying listicon requires scrolling.
    MapWindowPoints_(0,GadgetID(#Track_List),pt,1)
    If pt\y <=20  ;Change 20 to match the header height.
      top = SendMessage_(GadgetID(#Track_List), #LVM_GETTOPINDEX, 0, 0)
      If top ;Scroll up.
        SendMessage_(GadgetID(#Track_List), #LVM_ENSUREVISIBLE, top-1, 0)
        UpdateWindow_(GadgetID(#Track_List))         
      EndIf
    ElseIf pt\y>=GadgetHeight(#Track_List)-GetSystemMetrics_(#SM_CYHSCROLL)-4
      top = SendMessage_(GadgetID(#Track_List), #LVM_GETTOPINDEX, 0, 0)+SendMessage_(GadgetID(#Track_List), #LVM_GETCOUNTPERPAGE, 0, 0)
      If top<=CountGadgetItems(#Track_List) ;Scroll down.
        SendMessage_(GadgetID(#Track_List), #LVM_ENSUREVISIBLE, top, 0)
        UpdateWindow_(GadgetID(#Track_List))         
      EndIf
    EndIf
  EndIf
  ProcedureReturn #True
EndProcedure

Procedure.i Get_Seconds(time.s,withhour.b)
  
  Protected total,hour,min,sec,seconds
  
  If withhour : hour=0 : EndIf
  min=0
  sec=0
  
  If withhour : hour+Val(StringField(time,1,":")) : EndIf
  min+Val(StringField(time,2,":"))
  sec+Val(StringField(time,3,":"))
  
  If withhour : seconds=hour*3600 : EndIf
  seconds+(min*60)
  seconds+sec
  
  ProcedureReturn(seconds)  
  
EndProcedure

Procedure.s Time_String(s_total.s,withhour.b)
  
  Protected hour,min,sec,total,pad
  total=Val(s_total)
  hour=(total/3600)
  total=(total-(hour*3600))
  min=Int(total/60)
  sec=total-(min*60)
  
  pad=2
  
  If hour>99 : pad=3 : EndIf
  If hour>999 : pad=4 : EndIf
  If hour>9999 : pad=5 : EndIf
  
  If withhour
    ProcedureReturn(RSet(Str(hour),pad,"0")+":"+RSet(Str(min),2,"0")+":"+RSet(Str(sec),2,"0"))  
  Else
    ProcedureReturn(RSet(Str(min),2,"0")+":"+RSet(Str(sec),2,"0"))  
  EndIf
  
EndProcedure    

Procedure.s TotalTime()
  
  Protected total,hour,min,sec,pad
  
  hour=0
  min=0
  sec=0
  
  ForEach tracks()
    output=tracks()\t_time
    hour+Val(StringField(output,1,":"))
    min+Val(StringField(output,2,":"))
    sec+Val(StringField(output,3,":"))
  Next
  
  total=(hour*3600)+(min*60)+sec
  
  hour=(total/3600)
  total=(total-(hour*3600))
  min=Int(total/60)
  sec=total-(min*60)
  
  pad=2
  
  If hour>99 : pad=3 : EndIf
  If hour>999 : pad=4 : EndIf
  If hour>9999 : pad=5 : EndIf
  
  ProcedureReturn(RSet(Str(hour),pad,"0")+":"+RSet(Str(min),2,"0")+":"+RSet(Str(sec),2,"0"))  
  
EndProcedure

Procedure.l SearchDirectory(dir$, List yourLinkedList.s(), pattern$="", recursive=1)
  
  Protected n
  Static level.l=-1
  If Not Right(dir$, 1) = "\"
    dir$+"\"
  EndIf
  Protected dir.l=ExamineDirectory(#PB_Any, dir$, pattern$)
  If dir
    While NextDirectoryEntry(dir)
      If DirectoryEntryName(dir) <> "." And DirectoryEntryName(dir) <> ".."
        AddElement(yourLinkedList())
        For n=CountString(dir$, "\")-level To CountString(dir$, "\")
          yourLinkedList()+StringField(dir$, n, "\")+"\"
        Next
        StatusBarText(#Main_Status,0,"Searching... "+yourLinkedList(),#PB_StatusBar_Center)
        WindowEvent()
        yourLinkedList()+DirectoryEntryName(dir)
        If DirectoryEntryType(dir) = #PB_DirectoryEntry_Directory
          yourLinkedList()+"\"
        EndIf
      EndIf
    Wend
    FinishDirectory(dir)
  EndIf
  Protected all.l=ExamineDirectory(#PB_Any, dir$, "")
  If all
    While NextDirectoryEntry(all)
      If DirectoryEntryType(all) = #PB_DirectoryEntry_Directory And DirectoryEntryName(all) <> "." And DirectoryEntryName(all) <> ".."
        level+1
        SearchDirectory(dir$+DirectoryEntryName(all)+"\", yourLinkedList(), pattern$, recursive)
        level-1
      EndIf
    Wend
    FinishDirectory(all)
  EndIf
  ProcedureReturn ListSize(yourLinkedList())
  ClearList(yourLinkedList())
  
EndProcedure

Procedure MP3_Info(file.s)
  
  TR = TagReader::New_MP3TagObj(file)
  If TR
    tList = TR\GetTagList()    
    c = CountString(tList, #LF$)
    For i=1 To c+1
      
      Select StringField(tList, i, #LF$)
          
        Case "TITLE" : tracks()\t_title=TR\GetContent(StringField(tList, i, #LF$))
          
        Case "TIT2" : tracks()\t_title=TR\GetContent(StringField(tList, i, #LF$))
          
        Case "ARTIST" : tracks()\t_artist=TR\GetContent(StringField(tList, i, #LF$))
          
        Case "TPE1" :  tracks()\t_artist=TR\GetContent(StringField(tList, i, #LF$))
          
        Case "TBPM" :  tracks()\t_bpm=TR\GetContent(StringField(tList, i, #LF$))
          
        Case "LENGTH" : tracks()\t_time=TR\GetContent(StringField(tList, i, #LF$))
          
        Case "BITRATE" : tracks()\t_bitrate=TR\GetContent(StringField(tList, i, #LF$))
          
        Case "BITTYPE" : tracks()\t_vbr=TR\GetContent(StringField(tList, i, #LF$))
          
      EndSelect
    Next
    If capital
      tracks()\t_title=CapitalizeString(tracks()\t_title)
      tracks()\t_artist=CapitalizeString(tracks()\t_artist)
    EndIf
    TR\Destroy()
  EndIf
  
EndProcedure

Procedure MMP_Reader(mfile.s)
  
  Protected MMPFile,mpos,msize,i,mlength
  Protected *mmem,*mtext
  Protected out.s
  
  MMPFile=ReadFile(#PB_Any, mfile ,#PB_File_SharedRead)
  
  If MMPFile  
    mpos=0
    mlength=Lof(MMPFile)
    *mmem=AllocateMemory(mlength)
    ReadData(MMPFile,*mmem,mlength)
    Repeat
      If PeekS(*mmem+mpos,4,#PB_Ascii)="TRKF" 
        msize=0
        mpos+4 ; Move past header
        msize=Asc(PeekS(*mmem+mpos,4))
        If msize > 0
          mpos+4 ; Move past size info
          *mtext=AllocateMemory(msize)
          out=PeekS(*mmem+mpos, msize)
          PokeS(*mtext,out)
        EndIf
        AddElement(tracks())
        tracks()\t_path=PeekS(*mtext,msize)
        StatusBarText(#Main_Status,0,"Adding... "+tracks()\t_path,#PB_StatusBar_Center)

        mpos+msize
      EndIf
      mpos+1
    Until mpos>=Lof(MMPFile)
    FreeMemory(*mmem)
    FreeMemory(*mtext)
    CloseFile(MMPFile)
  EndIf  
  
EndProcedure

Procedure Menu_Check()
  If ListSize(tracks())>0
    DisableGadget(#Tool_SaveM3U,0)
    DisableGadget(#Tool_SaveSplit,0)
    DisableGadget(#Tool_Cut,0)
    DisableGadget(#Tool_Copy,0)
    DisableGadget(#Tool_Edit,0)
    DisableGadget(#Tool_Find,0)
    DisableGadget(#Tool_AutoFind,0)
    DisableGadget(#Tool_Clear,0)
    If cut="" : DisableGadget(#Tool_Paste,1) : EndIf
    If cut>"" : DisableGadget(#Tool_Copy,0) : EndIf
    If cut>"" : DisableGadget(#Tool_Paste,0) : EndIf
    DisableGadget(#Tool_Play,0)
    If IsMovie(#MP3_Play_File)=0 : DisableGadget(#Tool_Pause,1) : EndIf
    If IsMovie(#MP3_Play_File)=0 : DisableGadget(#Tool_Stop,1) : EndIf
    If IsMovie(#MP3_Play_File)=0 : DisableGadget(#Tool_Seek,1) : EndIf
    If IsMovie(#MP3_Play_File)=0 : DisableGadget(#Tool_Volume,1) : EndIf
    If IsMovie(#MP3_Play_File)>0 : DisableGadget(#Tool_Pause,0) : EndIf
    If IsMovie(#MP3_Play_File)>0 : DisableGadget(#Tool_Seek,0) : EndIf
    If IsMovie(#MP3_Play_File)>0 : DisableGadget(#Tool_Stop,0) : EndIf
    If IsMovie(#MP3_Play_File)>0 : DisableGadget(#Tool_Volume,0) : EndIf
    If IsMovie(#MP3_Play_File)=0 : DisableMenuItem(#Main_Menu,71,1) : EndIf
    If IsMovie(#MP3_Play_File)=0 : DisableMenuItem(#Main_Menu,72,1) : EndIf
    If IsMovie(#MP3_Play_File)>0 : DisableMenuItem(#Main_Menu,71,0) : EndIf
    If IsMovie(#MP3_Play_File)>0 : DisableMenuItem(#Main_Menu,72,0) : EndIf
    DisableMenuItem(#Main_Menu,2,0)
    DisableMenuItem(#Main_Menu,3,0)
    DisableMenuItem(#Main_Menu,4,0)
    DisableMenuItem(#Main_Menu,5,0)
    DisableMenuItem(#Main_Menu,6,0)
    DisableMenuItem(#Main_Menu,10,0)
    DisableMenuItem(#Main_Menu,11,0)
    DisableMenuItem(#Main_Menu,12,0)
    DisableMenuItem(#Main_Menu,13,0)
    DisableMenuItem(#Main_Menu,14,0)
    DisableMenuItem(#Main_Menu,15,0)
    If cut>"" : DisableMenuItem(#Main_Menu,16,0) : EndIf
    If cut>"" : DisableMenuItem(#Main_Menu,17,0) : EndIf
    DisableMenuItem(#Main_Menu,18,0)
    If ListSize(undo())>0 : DisableMenuItem(#Main_Menu,19,0) :EndIf
    DisableMenuItem(#Main_Menu,40,0)
    DisableMenuItem(#Main_Menu,41,0)
    DisableMenuItem(#Main_Menu,42,0)
    DisableMenuItem(#Main_Menu,43,0)
    DisableMenuItem(#Main_Menu,44,0)
    DisableMenuItem(#Main_Menu,45,0)
    DisableMenuItem(#Main_Menu,46,0)
    DisableMenuItem(#Main_Menu,47,0)
    DisableMenuItem(#Main_Menu,48,0)
    DisableMenuItem(#Main_Menu,49,0)
    DisableMenuItem(#Main_Menu,50,0)
    DisableMenuItem(#Main_Menu,61,0)
    DisableMenuItem(#Main_Menu,70,0)
    DisableMenuItem(#Main_Menu,73,0)
    DisableMenuItem(#Main_Menu,74,0)
    DisableMenuItem(#Main_Menu,75,0)
    DisableMenuItem(#Main_Menu,76,0)
    DisableMenuItem(#Main_Menu,22,0)
    DisableMenuItem(#Main_Menu,25,0)
    DisableMenuItem(#Main_Popup_Menu,10,0)
    DisableMenuItem(#Main_Popup_Menu,11,0)
    DisableMenuItem(#Main_Popup_Menu,13,0)
    DisableMenuItem(#Main_Popup_Menu,14,0)
    DisableMenuItem(#Main_Popup_Menu,15,0)
    If cut>"" : DisableMenuItem(#Main_Popup_Menu,16,0) : EndIf
    If cut>"" : DisableMenuItem(#Main_Popup_Menu,17,0) : EndIf
    DisableMenuItem(#Main_Popup_Menu,18,0)
    DisableMenuItem(#Main_Popup_Menu,22,0)
  Else
    DisableGadget(#Tool_SaveM3U,1)
    DisableGadget(#Tool_SaveSplit,1)
    DisableGadget(#Tool_Cut,1)
    DisableGadget(#Tool_Edit,1)
    DisableGadget(#Tool_Seek,1)
    DisableGadget(#Tool_Find,1)
    DisableGadget(#Tool_AutoFind,1)
    DisableGadget(#Tool_Clear,1)
    DisableGadget(#Tool_Copy,1)
    If cut="" : DisableGadget(#Tool_Paste,1) : EndIf
    DisableGadget(#Tool_Play,1)
    If IsMovie(#MP3_Play_File)=0 : DisableGadget(#Tool_Pause,1) : EndIf
    If IsMovie(#MP3_Play_File)=0 : DisableGadget(#Tool_Stop,1) : EndIf
    If IsMovie(#MP3_Play_File)=0 : DisableGadget(#Tool_Volume,1) : EndIf
    DisableMenuItem(#Main_Menu,2,1)
    DisableMenuItem(#Main_Menu,3,1)
    DisableMenuItem(#Main_Menu,4,1)
    DisableMenuItem(#Main_Menu,5,1)
    DisableMenuItem(#Main_Menu,6,1)
    DisableMenuItem(#Main_Menu,10,1)
    DisableMenuItem(#Main_Menu,11,1)
    DisableMenuItem(#Main_Menu,12,1)
    DisableMenuItem(#Main_Menu,13,1)
    DisableMenuItem(#Main_Menu,14,1)
    DisableMenuItem(#Main_Menu,15,1)
    DisableMenuItem(#Main_Menu,16,1)
    DisableMenuItem(#Main_Menu,17,1)
    DisableMenuItem(#Main_Menu,18,1)
    DisableMenuItem(#Main_Menu,19,1)
    DisableMenuItem(#Main_Menu,40,1)
    DisableMenuItem(#Main_Menu,41,1)
    DisableMenuItem(#Main_Menu,42,1)
    DisableMenuItem(#Main_Menu,43,1)
    DisableMenuItem(#Main_Menu,44,1)
    DisableMenuItem(#Main_Menu,45,1)
    DisableMenuItem(#Main_Menu,46,1)
    DisableMenuItem(#Main_Menu,47,1)
    DisableMenuItem(#Main_Menu,48,1)
    DisableMenuItem(#Main_Menu,49,1)
    DisableMenuItem(#Main_Menu,50,1)
    DisableMenuItem(#Main_Menu,61,1)
    DisableMenuItem(#Main_Menu,70,1)
    DisableMenuItem(#Main_Menu,71,1)
    DisableMenuItem(#Main_Menu,72,1)
    DisableMenuItem(#Main_Menu,73,1)
    DisableMenuItem(#Main_Menu,74,1)
    DisableMenuItem(#Main_Menu,75,1)
    DisableMenuItem(#Main_Menu,76,1)
    DisableMenuItem(#Main_Menu,22,1)
    DisableMenuItem(#Main_Menu,25,1)
    DisableMenuItem(#Main_Popup_Menu,10,1)
    DisableMenuItem(#Main_Popup_Menu,11,1)
    DisableMenuItem(#Main_Popup_Menu,13,1)
    DisableMenuItem(#Main_Popup_Menu,14,1)
    DisableMenuItem(#Main_Popup_Menu,15,1)
    DisableMenuItem(#Main_Popup_Menu,16,1)
    DisableMenuItem(#Main_Popup_Menu,17,1)
    DisableMenuItem(#Main_Popup_Menu,18,1)
    DisableMenuItem(#Main_Popup_Menu,22,1)
  EndIf
EndProcedure

Procedure Refresh_List()
  
  If ListSize(tracks())>0
    
    Protected oldlocation
    Protected KeyPressed$
    
    f=100/ListSize(tracks())
    j=0
    oldlocation=GetGadgetState(#Track_List)
    
    SendMessage_(GadgetID(#Track_List),#WM_SETREDRAW,#False,0)  
    ClearGadgetItems(#Track_List)
    
    ForEach tracks()
      j+f    
      If times
        If tracks()\t_artist="" 
          If FileSize(tracks()\t_path)>0
            StatusBarText(#Main_Status,0,"Scanning... "+tracks()\t_path)
            MP3_Info(tracks()\t_path)
            SetActiveWindow(#Main_Window)
            SetActiveGadget(#Track_List)
            StatusBarProgress(#Main_Status,1,j,#PB_StatusBar_Raised)
            WaitWindowEvent(1)
            event=WindowEvent()   
            If event = #WM_KEYDOWN And EventwParam() = #VK_ESCAPE : Break : EndIf
          EndIf
        EndIf
      EndIf
      output=Str(ListIndex(tracks()))+Chr(10)
      output+tracks()\t_path+Chr(10)
      output+tracks()\t_artist+Chr(10)
      output+tracks()\t_title+Chr(10)
      output+tracks()\t_bpm+Chr(10)
      output+tracks()\t_time+Chr(10)
      output+tracks()\t_bitrate+" ("+tracks()\t_vbr+")"
      AddGadgetItem(#Track_List,-1,output)
      If tracks()\t_artist="" : SetGadgetItemColor(#Track_List,CountGadgetItems(#Track_List)-1,#PB_Gadget_FrontColor,#Red,#PB_All) : EndIf
    Next
    
    StatusBarProgress(#Main_Status,1,0,#PB_StatusBar_Raised)
    SetGadgetState(#Track_List, 0) ; Reset listicon to 0...
    SetGadgetState(#Track_List, (oldlocation-1) + 15) ; Push our selection up to 'center'... [ the + 7 can be adjusted relative to listicon height ]
    If TargetItem>DragItem : SetGadgetState(#Track_List, oldlocation-1) : Else : SetGadgetState(#Track_List, oldlocation) : EndIf ; Highlight our selection
    
    SendMessage_(GadgetID(#Track_List),#WM_SETREDRAW,#True,0)
    RedrawWindow_(GadgetID(#Track_List), #Null, #Null, #RDW_ERASE | #RDW_FRAME | #RDW_INVALIDATE | #RDW_ALLCHILDREN)
    
    StatusBarText(#Main_Status,0,GetFilePart(m3ufile),#PB_StatusBar_Center)
    StatusBarText(#Main_Status,2,Str(ListSize(tracks()))+" Files",#PB_StatusBar_Center)
    StatusBarText(#Main_Status,3,TotalTime(),#PB_StatusBar_Center)
    SetActiveGadget(#Track_List)
    SetGadgetState(#Track_List,0)
    Menu_Check()
    
  EndIf

EndProcedure

Procedure Auto_Search()
  
  Global NewList FilesAndFolders.s()
  Protected NewMap SearchMap.s()
  
  path=PathRequester("Search Path","")
  StatusBarText(#Main_Status,0,"Searching... "+path,#PB_StatusBar_Center)
  If path
    ForEach tracks()
      If tracks()\t_artist=""
        SearchMap(GetFilePart(tracks()\t_path))=Str(ListIndex(tracks()))
      EndIf
    Next
    If MapSize(SearchMap())>0
      i=SearchDirectory(path, FilesAndFolders(), "", 1)
      ForEach FilesAndFolders()
        If FindMapElement(SearchMap(),GetFilePart(GetFilePart(FilesAndFolders())))
          SelectElement(tracks(),Val(SearchMap()))
          tracks()\t_path=path+FilesAndFolders()
        EndIf
      Next
    EndIf
  EndIf  

  FreeList(FilesAndFolders())
  FreeMap(SearchMap())
  Refresh_List()

EndProcedure

Procedure About_Window()
  Protected.s text
  Protected edit, w_event,font
  
  If OpenWindow(#Edit_Window,0,0,600,400,"About "+prog_title,#PB_Window_WindowCentered|#PB_Window_SystemMenu)
    
    oldgadgetlist=UseGadgetList(WindowID(#Edit_Window))
    
    font=LoadFont(#PB_Any,"Segoe",9)
    edit=EditorGadget(#PB_Any,0,0,WindowWidth(#Edit_Window),WindowHeight(#Edit_Window),#PB_Editor_ReadOnly|#PB_Editor_WordWrap)
    SetGadgetFont(edit,FontID(font))
    
    text=""
    text+prog_title+#LF$+#LF$
    text+Chr(169)+" Paul Vince 2016"+#LF$+#LF$
    text+"A small program to help compile play lists for music programs and generate lists for MixCloud. This program will load m3u & m3u8 files as well as Mixmeister playlists."
    text+"You can also create play lists from folders of MP3s as well a dropping files from explorer onto the target icon."+#LF$+#LF$
    text+"Once loaded you can filter, sort and remove duplicates entries. You can also search for missing files and manually enter MP3 details. At no time will this program write anything to your mp3 files! "
    text+"There is also a basic media player to preview the loaded MP3s"+#LF$+#LF$
    text+"Finally you can save new M3U files as well as splitting your playlists over multiple files. You can also save as a plain text file of artist and title for loading into MixCloud and other sites/programs."+#LF$+#LF$
    text+"DISCLAIMER"+#LF$+#LF$
    text+"This SOFTWARE PRODUCT is provided by THE PROVIDER "+Chr(34)+"As is"+Chr(34)+" And With "+Chr(34)+"all faults."+Chr(34)+" THE PROVIDER makes no representations Or warranties of any kind concerning the safety, suitability, lack of viruses, "
    text+"inaccuracies, typographical errors, Or other harmful components of this SOFTWARE PRODUCT. There are inherent dangers in the use of any software, And you are solely responsible For determining whether this SOFTWARE PRODUCT is compatible "
    text+"With your equipment And other software installed on your equipment. You are also solely responsible For the protection of your equipment And backup of your Data, And THE PROVIDER will Not be liable For any damages you may suffer in "
    text+"connection With using, modifying, Or distributing this SOFTWARE PRODUCT."+#LF$
    
    SetGadgetText(edit,text)
    
    Repeat
      w_event=WaitWindowEvent()   
    Until w_event=#PB_Event_CloseWindow
    
    UseGadgetList(oldgadgetlist)
    
    CloseWindow(#Edit_Window)
    FreeFont(font)
  EndIf
EndProcedure

Procedure Create_Split_List(s_type.i)
  
  ClearGadgetItems(#Split_List)
  ClearGadgetItems(#Split_Preview_Editor)  
  ClearList(split())
  
  Protected e_seconds, padding
  Protected exit=0
  Protected.s sp_name,sp_delim,sp_space1,sp_space2
  Protected s_seconds,splitnum,f_count
  Protected.s s_out1, s_out2
  
  sp_name=s_file
  
  
  If s_separator=0 : sp_delim="":EndIf
  If s_separator=1 : sp_delim="-":EndIf
  If s_separator=2 : sp_delim="_":EndIf
  If s_spaces=0 
    sp_space1="" : sp_space2=""
  Else 
    If sp_delim=""
      sp_space1=" " : sp_space2=""
    Else
      sp_space1=" " : sp_space2=" "
    EndIf
  EndIf
  padding=s_numbering+1
  
  s_time=GetGadgetState(#Split_Time)
  s_seconds=Get_Seconds(GetGadgetItemText(#Split_Time,s_time),#True)
  splitnum=s_filenum
  
  e_seconds=0
  count=0   
  f_count=0
  
  AddElement(split())                                                               ; Add Element to split list    
  split()\sp_name=sp_name+sp_space1+sp_delim+sp_space2+RSet(Str(ListSize(split())),padding,"0") ; Add split filename to list
  
  If s_type=0
    
    ForEach tracks()                                                                ; Loop through tracks list
      AddElement(split()\sp_file())                                                 ; Add element to split tracks list
      split()\sp_file()=tracks()\t_path                                             ; Add current track to split file list
      e_seconds+Get_Seconds(tracks()\t_time,#False)                                 ; Add track time to time count         
      split()\sp_length=Str(e_seconds)                                              ; Add track time to tracks list
      If e_seconds>s_seconds                                                        ; If the elapsed seconds are under split seconds 
        e_seconds-Get_Seconds(tracks()\t_time,#False)                               ; Remove last songs time so list is under required time
        s_out1=split()\sp_file()                                                    ; Remember Last Song
        s_out2=split()\sp_length                                                    ; Remember Song length
        DeleteElement(split()\sp_file())                                            ; Delete last song entry
        split()\sp_length=Str(e_seconds)                                            ; Add elasped time to list
        AddElement(split())                                                         ; Add a new element to the split list
        split()\sp_name=sp_name+sp_space1+sp_delim+sp_space2+RSet(Str(ListSize(split())),padding,"0") ; Add split filename to list
        AddElement(split()\sp_file())                                               ; Add element To split tracks List
        split()\sp_file()=s_out1                                                    ; Add Remembered Song
        split()\sp_length=s_out2                                                    ; Add Remembered Time
        e_seconds=0                                                                 ; Reset elapsed seconds
      EndIf                                         
    Next                                                                            ; Keep looping until end of list
    
  Else
    
    ForEach tracks()
      AddElement(split()\sp_file())                                                 ; Add element to split tracks list
      split()\sp_file()=tracks()\t_path                                             ; Add current track to split file list
      e_seconds+Get_Seconds(tracks()\t_time,#False)                                 ; Add track time to time count          
      split()\sp_length=Str(e_seconds)                                              ; Add track time to tracks list
      count+1                                                                       ; Add one to count for filename
      If count=splitnum                                                             ; If the count = the split number 
        split()\sp_length=Str(e_seconds)                                            ; Add total seconds to split list
        AddElement(split())                                                         ; Add a new element to the split list
        split()\sp_name=sp_name+sp_space1+sp_delim+sp_space2+RSet(Str(ListSize(split())),padding,"0") ; Add split filename to list
        e_seconds=0                                                                                 ; Reset elapsed seconds
        count=0                                                                                     ; Reset Count
      EndIf 
    Next                                                                             ; Keep looping until end of list
    
  EndIf
  
  SendMessage_(GadgetID(#Split_List),#WM_SETREDRAW,#False,0)  
  
  output=""
  
  ForEach split()
    If ListSize(split()\sp_file())=0 : DeleteElement(Split()) : Break : EndIf
    AddGadgetItem(#Split_List,-1,split()\sp_name+Chr(10)+Str(ListSize(split()\sp_file()))+Chr(10)+Time_String(split()\sp_length,#True))
    output+split()\sp_name+" - "+Time_String(split()\sp_length,#True)+#LF$
    ForEach split()\sp_file()
      output+#TAB$+split()\sp_file()+#LF$
    Next
    output+#LF$
  Next 
  
  AddGadgetItem(#Split_Preview_Editor,-1,output)
  
  SendMessage_(GadgetID(#Split_List),#WM_SETREDRAW,#True,0)
  RedrawWindow_(GadgetID(#Split_List), #Null, #Null, #RDW_ERASE | #RDW_FRAME | #RDW_INVALIDATE | #RDW_ALLCHILDREN)
  
EndProcedure

Procedure Split_Window()
  
  Protected.s s_out1,s_out2,s_out3,s_out4
  Protected split_type=0
  Protected.i s_len,s_num
  If m3ufile>"" : s_file=GetFilePart(m3ufile,#PB_FileSystem_NoExtension) : Else : s_file="File":EndIf
  s_out1="-"
  s_out2="0"
  s_out3=""
  s_out4=""
  s_len=1 ; bool for tickboxes
  s_num=0 ; bool for tickboxes
  
  If OpenWindow(#Split_Window,0,0,350,480,"Save Split M3U Files",#PB_Window_ScreenCentered|#PB_Window_SystemMenu)
    oldgadgetlist=UseGadgetList(WindowID(#Split_Window))
    FrameGadget(#PB_Any,5,5,340,25,"",#PB_Frame_Single)
    TextGadget(#PB_Any,10,43,100,15,"Individual File Name:")
    StringGadget(#Split_Name,120,40,140,20,s_file)
    ButtonGadget(#Split_Save,265,40,75,20,"Save")
    TextGadget(#PB_Any,10,73,50,15,"Separator")
    ComboBoxGadget(#Split_Separator,60,70,50,20)
    AddGadgetItem(#Split_Separator,0,"None")
    AddGadgetItem(#Split_Separator,1,"-")
    AddGadgetItem(#Split_Separator,2,"_")
    SetGadgetState(#Split_Separator,s_separator)
    TextGadget(#PB_Any,120,73,50,15,"Numbering")
    ComboBoxGadget(#Split_Numbering,180,70,50,20)
    AddGadgetItem(#Split_Numbering,0,"#")
    AddGadgetItem(#Split_Numbering,1,"##")
    AddGadgetItem(#Split_Numbering,2,"###")
    SetGadgetState(#Split_Numbering,s_numbering)
    CheckBoxGadget(#Split_Spaces,250,73,80,15,"Add Spaces")
    SetGadgetState(#Split_Spaces,s_spaces)
    CheckBoxGadget(#Split_Split_Len,10,103,90,15,"Split by Length")
    ComboBoxGadget(#Split_Time,105,100,70,20)
    AddGadgetItem(#Split_Time,0,"00:15:00")
    AddGadgetItem(#Split_Time,1,"00:30:00")
    AddGadgetItem(#Split_Time,2,"00:45:00")
    AddGadgetItem(#Split_Time,3,"01:00:00")
    AddGadgetItem(#Split_Time,4,"01:30:00")
    AddGadgetItem(#Split_Time,5,"02:00:00")
    AddGadgetItem(#Split_Time,6,"02:30:00")
    AddGadgetItem(#Split_Time,7,"03:00:00")
    AddGadgetItem(#Split_Time,8,"03:30:00")
    AddGadgetItem(#Split_Time,9,"04:00:00")
    AddGadgetItem(#Split_Time,10,"04:30:00")
    AddGadgetItem(#Split_Time,11,"05:00:00")
    AddGadgetItem(#Split_Time,12,"05:30:00")
    AddGadgetItem(#Split_Time,13,"06:00:00")
    AddGadgetItem(#Split_Time,14,"06:30:00")
    AddGadgetItem(#Split_Time,15,"07:00:00")
    AddGadgetItem(#Split_Time,16,"07:30:00")
    AddGadgetItem(#Split_Time,17,"08:00:00")
    AddGadgetItem(#Split_Time,18,"08:30:00")
    AddGadgetItem(#Split_Time,19,"09:00:00")
    AddGadgetItem(#Split_Time,20,"09:30:00")
    AddGadgetItem(#Split_Time,21,"10:00:00")
    AddGadgetItem(#Split_Time,22,"10:30:00")
    AddGadgetItem(#Split_Time,23,"11:00:00")
    AddGadgetItem(#Split_Time,24,"11:30:00")
    AddGadgetItem(#Split_Time,25,"12:00:00")
    SetGadgetState(#Split_Time,s_time)
    CheckBoxGadget(#Split_Split_Num,185,100,85,20,"Split By Files")
    SpinGadget(#Split_Files,270,100,70,20,1,200,#PB_Spin_Numeric)
    SetGadgetState(#Split_Files,s_filenum)
    Select s_separator
      Case 0 : s_out1=""
      Case 1 : s_out1="-"
      Case 2 : s_out1="_"
    EndSelect
    Select s_numbering
      Case 0 : s_out2="0"
      Case 1 : s_out2="00"
    EndSelect
    
    PanelGadget(#Split_Panel,5,130,340,345)
    AddGadgetItem(#Split_Panel,0,"File List")
    ListIconGadget(#Split_List,0,0,GadgetWidth(#Split_Panel)-5,GadgetHeight(#Split_Panel)-25,"Name Preview",210,#PB_ListIcon_GridLines)
    AddGadgetColumn(#Split_List,1,"# Entries",60)
    AddGadgetColumn(#Split_List,2,"Length",60)
    AddGadgetItem(#Split_Panel,1,"Preview")
    EditorGadget(#Split_Preview_Editor,0,0,GadgetWidth(#Split_Panel)-5,GadgetHeight(#Split_Panel)-25,#PB_Editor_ReadOnly)
    CloseGadgetList()
    
    TextGadget(#Split_Example,10,10,330,15,s_file+s_out3+s_out1+s_out3+s_out2,#PB_Text_Center)
    
    If split_type=1 : SetGadgetState(#Split_Split_Len,#False) : Else : SetGadgetState(#Split_Split_Len,#True):EndIf
    If split_type=1 : DisableGadget(#Split_Files,#False) : Else : DisableGadget(#Split_Files,#True):EndIf
    If split_type=0 : SetGadgetState(#Split_Split_Num,#False):Else: SetGadgetState(#Split_Split_Num,#True):EndIf
    If split_type=0 : DisableGadget(#Split_Time,#False):Else: SetGadgetState(#Split_Time,#True):EndIf
    
    Create_Split_List(s_type)
    
    Repeat
      event=WaitWindowEvent()
      type=EventType()
      gadget=EventGadget()
      
      Select gadget
          
        Case #Split_Save
          path=PathRequester("Save Path",GetCurrentDirectory())
          If path
            ForEach split()
              file=path+split()\sp_name+".m3u"
              CreateFile(#M3U_File,file,#PB_File_SharedRead|#PB_Ascii)
              ForEach split()\sp_file()
                WriteStringN(#M3U_File,split()\sp_file(),#PB_Ascii)
              Next
              CloseFile(#M3U_File)
            Next
          EndIf
                    
        Case #Split_Files
          If type=#PB_EventType_Change
            s_filenum=GetGadgetState(#Split_Files)
            SetGadgetText(#Split_Example,s_file+s_out3+s_out1+s_out4+s_out2)
            Create_Split_List(s_type)
          EndIf
          
        Case #Split_Name
          If type=#PB_EventType_Change
            s_file=GetGadgetText(#Split_Name)
            SetGadgetText(#Split_Example,s_file+s_out3+s_out1+s_out4+s_out2)
            Create_Split_List(s_type)
          EndIf
          
        Case #Split_Time
          If type=#PB_EventType_Change
            s_time=GetGadgetState(#Split_Time)
            SetGadgetText(#Split_Example,s_file+s_out3+s_out1+s_out4+s_out2)
            Create_Split_List(s_type)
          EndIf
          
        Case #Split_Spaces
          If s_spaces=0 
            SetGadgetState(#Split_Spaces,1)
            s_spaces=1
            If s_out1=""
              s_out3=" " : s_out4=""
            Else
              s_out3=" " : s_out4=" "
            EndIf
          Else
            SetGadgetState(#Split_Spaces,0)
            s_spaces=0
            s_out3="" : s_out4=""
          EndIf
          SetGadgetText(#Split_Example,s_file+s_out3+s_out1+s_out4+s_out2)
          Create_Split_List(s_type)
          
        Case #Split_Separator
          If type=#PB_EventType_Change
            s_separator=GetGadgetState(#Split_Separator)
            Select s_separator
              Case 0 
                s_out1=""
              Case 1
                s_out1="-"
              Case 2
                s_out1="_"
            EndSelect
            If s_spaces=1 
              If s_out1=""
                s_out3=" " : s_out4=""
              Else
                s_out3=" " : s_out4=" "
              EndIf
            Else
              s_out3="" : s_out4=""
            EndIf
            SetGadgetText(#Split_Example,s_file+s_out3+s_out1+s_out4+s_out2)
            Create_Split_List(s_type)       
          EndIf
          
        Case #Split_Numbering
          If type=#PB_EventType_Change
            s_numbering=GetGadgetState(#Split_Numbering)
            Select s_numbering
              Case 0 
                s_out2=RSet(s_out2,1,"0")
              Case 1
                s_out2=RSet(s_out2,2,"0")
              Case 2
                s_out2=RSet(s_out2,3,"0")
            EndSelect
            SetGadgetText(#Split_Example,s_file+s_out3+s_out1+s_out4+s_out2)
          EndIf
          Create_Split_List(s_type)
          
        Case #Split_Split_Len
          s_type=0
          s_time=GetGadgetState(#Split_Time)
          s_num=0
          DisableGadget(#Split_Files,1)
          DisableGadget(#Split_Time,0)
          SetGadgetState(#Split_Split_Num,s_num)
          Create_Split_List(s_type)
          
        Case #Split_Split_Num
          s_type=1
          s_filenum=GetGadgetState(#Split_Files)
          s_len=0
          DisableGadget(#Split_Files,0)
          DisableGadget(#Split_Time,1)
          SetGadgetState(#Split_Split_Len,s_len)
          Create_Split_List(s_type)
          
      EndSelect
      
    Until event=#PB_Event_CloseWindow And EventWindow()=#Split_Window
    
    WaitWindowEvent()
    UseGadgetList(oldgadgetlist)
    CloseWindow(#Split_Window)
    ClearList(split())
    
  EndIf
EndProcedure

Procedure Edit_Window()
  Protected.s e_artist, e_title,e_time,e_bpm,e_bitrate,e_bittype
  e_artist=tracks()\t_artist
  e_title=tracks()\t_title
  e_time=tracks()\t_time
  e_bpm=tracks()\t_bpm
  e_bitrate=tracks()\t_bitrate
  e_bittype=tracks()\t_vbr
  
  If OpenWindow(#Edit_Window,0,0,550,180,"Edit List",#PB_Window_ScreenCentered|#PB_Window_NoGadgets)
    oldgadgetlist=UseGadgetList(WindowID(#Edit_Window))
    TextGadget(#PB_Any,5,13,50,20,"File Path")
    StringGadget(#Edit_Path,55,10,490,20,tracks()\t_path,#PB_String_ReadOnly)
    TextGadget(#PB_Any,5,43,50,20,"Artist")
    StringGadget(#Edit_Artist,55,40,490,20,tracks()\t_artist)
    TextGadget(#PB_Any,5,73,50,20,"Title")
    StringGadget(#Edit_Title,55,70,490,20,tracks()\t_title)
    FrameGadget(#PB_Any,10,105,130,30,"",#PB_Frame_Flat)
    FrameGadget(#PB_Any,145,105,130,30,"",#PB_Frame_Flat)
    FrameGadget(#PB_Any,280,105,130,30,"",#PB_Frame_Flat)
    FrameGadget(#PB_Any,415,105,130,30,"",#PB_Frame_Flat)
    TextGadget(#PB_Any,15,113,60,15,"Time :",#PB_Text_Center)
    StringGadget(#Edit_Time,75,113,60,15,tracks()\t_time,#PB_String_BorderLess)
    TextGadget(#PB_Any,140,113,60,15,"BPM :",#PB_Text_Center)
    StringGadget(#Edit_BPM,200,113,60,15,tracks()\t_bpm,#PB_String_BorderLess)
    TextGadget(#PB_Any,285,113,60,15,"Bitrate :",#PB_Text_Center)
    StringGadget(#Edit_Bitrate,345,113,60,15,tracks()\t_bitrate,#PB_String_BorderLess)
    TextGadget(#PB_Any,420,113,60,15,"Bit Type :",#PB_Text_Center)
    StringGadget(#Edit_BitType,480,113,60,15,tracks()\t_vbr,#PB_String_BorderLess)
    ButtonGadget(#Edit_Close,65,145,200,30,"Save")
    ButtonGadget(#Edit_Cancel,295,145,200,30,"Cancel")
    Repeat
      event=WaitWindowEvent()
      type=EventType()
      gadget=EventGadget()
      
      Select gadget
          
        Case #Edit_Artist
          If type=#PB_EventType_Change
            e_artist=GetGadgetText(#Edit_Artist)
          EndIf
          
        Case #Edit_Title
          If type=#PB_EventType_Change
            e_title=GetGadgetText(#Edit_Title)
          EndIf
          
        Case #Edit_BPM
          If type=#PB_EventType_Change
            e_bpm=GetGadgetText(#Edit_BPM)
          EndIf
          
        Case #Edit_Bitrate
          If type=#PB_EventType_Change
            e_bitrate=GetGadgetText(#Edit_Bitrate)
          EndIf
          
        Case #Edit_BitType
          If type=#PB_EventType_Change
            e_bittype=GetGadgetText(#Edit_BitType)
          EndIf
          
        Case #Edit_Time
          If type=#PB_EventType_Change
            e_time=GetGadgetText(#Edit_Time)
          EndIf
          
        Case #Edit_Close
          result=MessageRequester("Edit List","Save Changes?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)
          If result=#PB_MessageRequester_Yes
            tracks()\t_artist=e_artist 
            tracks()\t_title=e_title
            tracks()\t_vbr=e_bittype
            tracks()\t_time=e_time
            tracks()\t_bitrate=e_bitrate
            tracks()\t_bpm=e_bpm
            i=1 
          EndIf
          
        Case #Edit_Cancel
          result=MessageRequester("Edit List","Lose Changes?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)
          If result=#PB_MessageRequester_Yes : i=1 : EndIf      
          
      EndSelect
    Until i=1
    UseGadgetList(oldgadgetlist)
    CloseWindow(#Edit_Window)
  EndIf
EndProcedure

Procedure Menus()
  
  If CreateMenu(#Main_Menu,WindowID(#Main_Window))
    MenuTitle("File")
    MenuItem(1,"&Open Playlist" + Chr(9) + "Ctrl+O")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_O,1)
    MenuItem(2,"&Append Playlist" + Chr(9) + "Ctrl+Shift+O")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Shift|#PB_Shortcut_O,2)
    MenuItem(8,"&Open Multiple Playlists" + Chr(9) + "Ctrl+M")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_M,2)
    MenuBar()
    MenuItem(9,"&Add MP3 Folder" + Chr(9) + "Ctrl+F")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_F,9)
    MenuBar() 
    MenuItem(3,"&Save M3U" + Chr(9) + "Ctrl+S")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_S,3)
    MenuItem(4,"&Save Split M3U" + Chr(9) + "Ctrl+Shift+S")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Shift|#PB_Shortcut_S,4)
    MenuItem(5,"&Save List" + Chr(9) + "Ctrl+L")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_L,5)
    MenuBar()
    MenuItem(60,"&Quickload Playlist" + Chr(9) + "Ctrl+Q")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Q,60)
    MenuItem(61,"&Save Quickload" + Chr(9) + "Ctrl+Shift+Q")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Shift|#PB_Shortcut_Q,61)
    MenuBar()
    MenuItem(6,"&Clear List" + Chr(9) + "Ctrl+Shift+X")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_X,6)
    MenuBar()
    MenuItem(7,"&Close" + Chr(9) + "Alt+F4")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Alt|#PB_Shortcut_F4,7)
    MenuTitle("Edit")
    MenuItem(19,"&Undo" + Chr(9) + "Ctrl+Z")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Z,19)
    MenuItem(15,"&Cut" + Chr(9) + "Ctrl+X")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_X,15)
    MenuItem(18,"&Copy" + Chr(9) + "Ctrl+C")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_C,18)
    MenuItem(16,"&Paste" + Chr(9) + "Ctrl+V")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_V,16)
    MenuItem(17,"Paste (&Last)" + Chr(9) + "Ctrl+Shift+V")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Shift|#PB_Shortcut_V,17)
    MenuItem(25,"&Delete" + Chr(9) + "Del")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Delete,25)
    MenuBar()
    MenuItem(14,"&Copy List To Clip" + Chr(9) + "Ctrl+A")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_A,14)
    
    MenuTitle("List")
    MenuItem(43,"Sort List By &Path" + Chr(9) + "Ctrl+1")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_1,43)
    MenuItem(40,"Sort List By &Artist" + Chr(9) + "Ctrl+2")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_2,40)
    MenuItem(41,"Sort List By &Title" + Chr(9) + "Ctrl+3")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_3,41)
    MenuItem(46,"Sort List By &BPM" + Chr(9) + "Ctrl+4")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_4,46)
    MenuItem(42,"Sort List By &Time" + Chr(9) + "Ctrl+5")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_5,42)
    MenuItem(50,"Sort List By &Bitrate" + Chr(9) + "Ctrl+6")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_6,50)
    MenuBar()
    MenuItem(49,"Sort Ascending" + Chr(9) + "Ctrl+0")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_0,49)
    MenuBar()
    MenuItem(47,"Remove &Dupes" + Chr(9) + "Ctrl+D")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_D,47)
    MenuItem(44,"&Randomize List" + Chr(9) + "Ctrl+R")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_R,44)
    MenuBar()
    MenuItem(48,"&Snapshot List" + Chr(9) + "Ctrl+B")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_B,48)
    MenuItem(45,"&Default List" + Chr(9) + "Ctrl+Shift+B")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Shift|#PB_Shortcut_B,45)
    
    MenuTitle("MP3")
    MenuItem(70,"&Play MP3 File" + Chr(9) + "Ctrl+P")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_P,70)
    MenuItem(71,"&Pause MP3 File" + Chr(9) + "Ctrl+Shift+P")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Shift|#PB_Shortcut_P,71)
    MenuItem(72,"&Stop MP3 File" + Chr(9) + "Ctrl+S")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_S,72)
    
    MenuBar()
    MenuItem(10,"&Find MP3 File" + Chr(9) + "Ctrl+F")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_F,10)
    MenuItem(76,"&Auto Find File" + Chr(9) + "Ctrl+Shift+F")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Shift|#PB_Shortcut_F,76)
    MenuItem(11,"Scan MP3 &Info" + Chr(9) + "Ctrl+I")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_I,11)
    MenuItem(12,"Scan &All Files" + Chr(9) + "Ctrl+Shift+I")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_Shift|#PB_Shortcut_I,12)
    MenuBar()
    MenuItem(13,"&Edit MP3 Info" + Chr(9) + "Ctrl+E")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_E,13)
    
    MenuTitle("Options")
    MenuItem(20,"&Capitalize First Letter" + Chr(9) + "Ctrl+L")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_L,20)
    SetMenuItemState(#Main_Menu,20,capital)
    MenuItem(21,"&Load With MP3 Info" + Chr(9) + "Ctrl+W")
    AddKeyboardShortcut(#MAIN_WINDOW,#PB_Shortcut_Control|#PB_Shortcut_W,21)
    SetMenuItemState(#Main_Menu,21,times)
    MenuBar()
    MenuItem(22,"&Refresh")
    MenuTitle("About")

    MenuItem(23,"&About")
  EndIf
  
  If CreatePopupMenu(#Main_Popup_Menu)
    MenuItem(15,"Cut")
    MenuItem(18,"Copy")
    MenuItem(16,"Paste")
    MenuItem(17,"Paste (Last)")
    MenuBar()
    MenuItem(10,"Find MP3 File")
    MenuItem(11,"Scan MP3 Info")
    MenuItem(13,"Edit MP3 Info")
  EndIf
  
EndProcedure

Procedure Gadgets()
  
  If CreateStatusBar(#Main_Status,WindowID(#Main_Window))
    AddStatusBarField(870)
    AddStatusBarField(180)
    AddStatusBarField(75)
    AddStatusBarField(75)  
  EndIf
  
  StatusBarText(#Main_Status,0,prog_title,#PB_StatusBar_Center)
  StatusBarText(#Main_Status,2,"0 Files",#PB_StatusBar_Center)
  StatusBarText(#Main_Status,3,"00:00:00",#PB_StatusBar_Center)
  StatusBarProgress(#Main_Status,1,0,#PB_StatusBar_Raised)
  
  ListIconGadget(#Track_List,0,30,WindowWidth(#Main_Window),575,"#",30,#PB_ListIcon_GridLines|#PB_ListIcon_FullRowSelect)
  AddGadgetColumn(#Track_List,1,"Filename",450)
  AddGadgetColumn(#Track_List,2,"Artist",250)
  AddGadgetColumn(#Track_List,3,"Title",250)
  AddGadgetColumn(#Track_List,4,"BPM",50)
  AddGadgetColumn(#Track_List,5,"Time",70)
  AddGadgetColumn(#Track_List,6,"Bitrate",70)
    
  ContainerGadget(#Tool_Container,0,0,WindowWidth(#Main_Window),30,#PB_Container_Single)
  CatchImage(#Tool_OpenM3U_img, ?OpenM3U)
  ResizeImage(#Tool_OpenM3U_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_OpenFolder_img,?OpenMMP)
  ResizeImage(#Tool_OpenFolder_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_SaveM3U_img,?SaveM3U)
  ResizeImage(#Tool_SaveM3U_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_SaveSplit_img,?SaveSplit)
  ResizeImage(#Tool_SaveSplit_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Cut_img,?Cut)
  ResizeImage(#Tool_Cut_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Copy_img,?Copy)
  ResizeImage(#Tool_Copy_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Paste_img,?Paste)
  ResizeImage(#Tool_Paste_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Edit_img,?Edit)
  ResizeImage(#Tool_Edit_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Find_img,?Find)
  ResizeImage(#Tool_Find_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_AutoFind_img,?AutoFind)
  ResizeImage(#Tool_AutoFind_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Play_img,?Play)
  ResizeImage(#Tool_Play_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Pause_img,?Pause)
  ResizeImage(#Tool_Pause_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Stop_img,?Stop)
  ResizeImage(#Tool_Stop_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Clear_img,?Clear)
  ResizeImage(#Tool_Clear_img,20,20,#PB_Image_Smooth)
  CatchImage(#Tool_Target_img,?Target)
  ResizeImage(#Tool_Target_img,20,20,#PB_Image_Smooth)
  ButtonImageGadget(#Tool_OpenM3U,0,0,30,29,ImageID(#Tool_OpenM3U_img))
  GadgetToolTip(#Tool_OpenM3U,"Open File")
  ButtonImageGadget(#Tool_OpenFolder,30,0,30,29,ImageID(#Tool_OpenFolder_img))
  GadgetToolTip(#Tool_OpenFolder,"Add Folder")
  ButtonImageGadget(#Tool_SaveM3U,65,0,30,29,ImageID(#Tool_SaveM3U_img))
  GadgetToolTip(#Tool_SaveM3U,"Save M3U File")
  ButtonImageGadget(#Tool_SaveSplit,95,0,30,29,ImageID(#Tool_SaveSplit_img))
  GadgetToolTip(#Tool_SaveSplit,"Save Split M3U Files")
  ButtonImageGadget(#Tool_Cut,130,0,30,29,ImageID(#Tool_Cut_img))
  GadgetToolTip(#Tool_Cut,"Cut")
  ButtonImageGadget(#Tool_Copy,160,0,30,29,ImageID(#Tool_Copy_img))
  GadgetToolTip(#Tool_Copy,"Copy")
  ButtonImageGadget(#Tool_Paste,190,0,30,29,ImageID(#Tool_Paste_img))
  GadgetToolTip(#Tool_Paste,"Paste")
  ButtonImageGadget(#Tool_Edit,225,0,30,29,ImageID(#Tool_Edit_img))
  GadgetToolTip(#Tool_Edit,"Edit")
  ButtonImageGadget(#Tool_Find,255,0,30,29,ImageID(#Tool_Find_img))
  GadgetToolTip(#Tool_Find,"Find File")
  ButtonImageGadget(#Tool_AutoFind,285,0,30,29,ImageID(#Tool_AutoFind_img))
  GadgetToolTip(#Tool_AutoFind,"Auto File Finder")
  ButtonImageGadget(#Tool_Play,320,0,30,29,ImageID(#Tool_Play_img))
  GadgetToolTip(#Tool_Play,"Play")
  ButtonImageGadget(#Tool_Pause,350,0,30,29,ImageID(#Tool_Pause_img))
  GadgetToolTip(#Tool_Pause,"Pause")
  ButtonImageGadget(#Tool_Stop,380,0,30,29,ImageID(#Tool_Stop_img))
  GadgetToolTip(#Tool_Stop,"Stop")
  TextGadget(#PB_Any,450,8,40,30,"Seek:",#PB_Text_Center)
  TrackBarGadget(#Tool_Seek,495,0,300,29,1,100,#PB_TrackBar_Ticks)
  TextGadget(#PB_Any,810,8,40,30,"Volume:",#PB_Text_Center)
  M_Volume=50
  TrackBarGadget(#Tool_Volume,855,0,300,29,0,100,#PB_TrackBar_Ticks)
  SetGadgetState(#Tool_Volume,M_Volume)
  ButtonImageGadget(#Tool_Clear,415,0,30,29,ImageID(#Tool_Clear_img))
  GadgetToolTip(#Tool_Clear,"Clear List")
  ImageGadget(#Tool_Target,1172,3,30,30,ImageID(#Tool_Target_img))
  GadgetToolTip(#Tool_Target,"Drop Files Here!")
  
  EnableGadgetDrop(#Track_List, #PB_Drop_Private, #PB_Drag_Move, 1)
  EnableGadgetDrop(#Tool_Target,#PB_Drop_Files,#PB_Drag_Copy)
  
  CloseGadgetList()
  
EndProcedure

Procedure MP3Drop()
  
  Global NewList FilesAndFolders.s()
  
  path = EventDropFiles()
  If GetExtensionPart(path)=""
    If path
      i=SearchDirectory(path, FilesAndFolders(), "", 1)
      ForEach FilesAndFolders()
        If LCase(GetExtensionPart(FilesAndFolders()))="mp3"
          LastElement(tracks())
          AddElement(tracks())
          tracks()\t_path=path+"\"+FilesAndFolders()
        EndIf
      Next
    EndIf  
    
    FreeList(FilesAndFolders())

  Else
    
    tcount  = CountString(path, Chr(10)) + 1
    
    For tc = 1 To tcount
      If LCase(GetExtensionPart(StringField(path, tc, Chr(10))))="mp3"
        LastElement(tracks())
        AddElement(tracks())
        tracks()\t_path=StringField(path, tc, Chr(10))
        StatusBarText(#Main_Status,0,"Adding... "+tracks()\t_path,#PB_StatusBar_Center)
      EndIf
    Next tc
  EndIf
  
  Refresh_List()  
  
EndProcedure

InitMovie()
UsePNGImageDecoder()

OpenWindow(#Main_Window,0,0,1200,650,prog_title,#PB_Window_ScreenCentered|#PB_Window_MinimizeGadget)

Gadgets()
Menus()
Menu_Check()

Repeat
  
  event=WaitWindowEvent()
  gadget=EventGadget()
  
  Select event
      
    Case #WM_RBUTTONDOWN 
      DisplayPopupMenu(#Main_Popup_Menu, WindowID(#Main_Window))     
            
    Case #PB_Event_GadgetDrop

      Select gadget
          
        Case #Track_List
          If EventDropPrivate() = 1 And GetGadgetState(#Track_List)>-1
            TargetItem = GetGadgetState(#Track_List)
            SelectElement(tracks(),TargetItem)
            *relative=@Tracks()
            SelectElement(Tracks(),DragItem)
            MoveElement(Tracks(), #PB_List_Before, *Relative)
            Refresh_List()
            UndoLoc=DragItem
            TargetItem=DragItem
          EndIf
          
        Case #Tool_Target
          If EventDropFiles()>""
            MP3Drop()
          EndIf

      EndSelect
      
    Case #PB_Event_Gadget
      
      Select EventType()
          
        Case #PB_EventType_LeftDoubleClick
          If EventGadget()=#Track_List
          PostEvent(#PB_Event_Menu, 0, 70)
          EndIf
          
        Case #PB_EventType_DragStart
          DragItem = GetGadgetState(#Track_List)
          SetActiveGadget(#Track_List)
          CopyList(tracks(),undo())   
          DragPrivate(1, #PB_Drag_Move)
          SetDragCallback(@DragCallBack())
          
      EndSelect
      
      Select EventGadget()
          
        Case #Tool_OpenM3U
          PostEvent(#PB_Event_Menu, 0, 1)
          
        Case #Tool_OpenFolder
          PostEvent(#PB_Event_Menu, 0, 9)
          
        Case #Tool_SaveM3U
          PostEvent(#PB_Event_Menu, 0, 3)
          
        Case #Tool_SaveSplit
          PostEvent(#PB_Event_Menu, 0, 4)
          
        Case #Tool_Cut
          PostEvent(#PB_Event_Menu, 0, 15)
          
        Case #Tool_Copy
          PostEvent(#PB_Event_Menu, 0, 18)
          
        Case #Tool_Paste
          PostEvent(#PB_Event_Menu, 0, 16)
          
        Case #Tool_Edit
          PostEvent(#PB_Event_Menu, 0, 13)
          
        Case #Tool_Find
          PostEvent(#PB_Event_Menu, 0, 10)
          
        Case #Tool_AutoFind
          PostEvent(#PB_Event_Menu, 0, 76)
          
        Case #Tool_Play
          PostEvent(#PB_Event_Menu, 0, 70)
          
        Case #Tool_Pause
          PostEvent(#PB_Event_Menu, 0, 71)
          
        Case #Tool_Stop
          PostEvent(#PB_Event_Menu, 0, 72)
          
        Case #Tool_Clear
          PostEvent(#PB_Event_Menu, 0, 6)
          
        Case #Tool_Volume
          M_Volume=GetGadgetState(#Tool_Volume)
          If mp3load 
            MovieAudio(#MP3_Play_File,M_Volume,0)
          EndIf
          
        Case #Tool_Seek
          i=GetGadgetState(#Tool_Seek)
          If IsMovie(#MP3_Play_File)
            mp3pos=Abs(MovieLength(#MP3_Play_File))/fps
            MovieSeek(#MP3_Play_File,mp3pos*i)
          EndIf
          
      EndSelect  
      
    Case #PB_Event_Menu      
      
      Select EventMenu()
          
        Case 1 ; Load File
          m3ufile=OpenFileRequester("Load PlayList","","M3U File (*.m3u)|*.m3u|MMP File (*.mmp)|*.mmp",0)
          If m3ufile<>""
            folder=GetPathPart(m3ufile)
            SetCurrentDirectory(folder)
            If OpenFile(#M3U_File,m3ufile)
              If ListSize(tracks())>-1 : ClearList(tracks()) : EndIf
              ClearGadgetItems(#Track_List)
              If LCase(GetExtensionPart(m3ufile))="m3u"
                While Not Eof(#M3U_File)
                  output=ReadString(#M3U_File,ReadStringFormat(#M3U_File))
                  If Left(output,1)<>"#"
                    AddElement(tracks())  
                    tracks()\t_path=output
                  EndIf
                Wend
              EndIf
              CloseFile(#M3U_File)              
              If LCase(GetExtensionPart(m3ufile))="mmp"     
                MMP_Reader(m3ufile)
              EndIf  
              If ListSize(tracks())>-1
                Refresh_List()
                CopyList(tracks(),backup())
              EndIf
            EndIf
          EndIf
          
        Case 2 ; Append File
          If ListSize(tracks())>0
            CopyList(tracks(),undo())
            m3ufile=OpenFileRequester("Append Playlist","","M3U File (*.m3u)|*.m3u|MMP File (*.mmp)|*.mmp",0)
            If m3ufile<>""
              folder=GetPathPart(m3ufile)
              SetCurrentDirectory(folder)
              If OpenFile(#M3U_File,m3ufile)
                If ListSize(tracks())>-1 : SelectElement(tracks(),ListSize(tracks())-1) : EndIf
                ClearGadgetItems(#Track_List)
                If LCase(GetExtensionPart(m3ufile))="m3u"
                  While Not Eof(#M3U_File)
                    output=ReadString(#M3U_File,ReadStringFormat(#M3U_File))
                    If Left(output,1)<>"#"
                      AddElement(tracks())  
                      tracks()\t_path=output
                    EndIf
                  Wend 
                EndIf
                CloseFile(#M3U_File)
                If LCase(GetExtensionPart(m3ufile))="mmp"     
                  MMP_Reader(m3ufile)
                EndIf
                If ListSize(tracks())>-1
                  Refresh_List()
                  CopyList(tracks(),backup())
                EndIf
              EndIf
            EndIf
          EndIf
          
        Case 8 ; Open Multiple Files
          m3ufile=OpenFileRequester("Open Multiple Playlists","","M3U File (*.m3u)|*.m3u|MMP File (*.mmp)|*.mmp",0,#PB_Requester_MultiSelection)
          If m3ufile<>""
            folder=GetPathPart(m3ufile)
            SetCurrentDirectory(folder)
            If ListSize(tracks())>-1 : ClearList(tracks()) : EndIf
            ClearGadgetItems(#Track_List)          
            While m3ufile
              If OpenFile(#M3U_File,m3ufile)
                If LCase(GetExtensionPart(m3ufile))="m3u"
                  While Not Eof(#M3U_File)
                    output=ReadString(#M3U_File,ReadStringFormat(#M3U_File))
                    If Left(output,1)<>"#"
                      AddElement(tracks())  
                      tracks()\t_path=output
                    EndIf
                  Wend 
                EndIf
                CloseFile(#M3U_File)
                If LCase(GetExtensionPart(m3ufile))="mmp"     
                  MMP_Reader(m3ufile)
                EndIf 
              EndIf
              m3ufile=NextSelectedFileName()
            Wend
            If ListSize(tracks())>-1
              Refresh_List()
              CopyList(tracks(),backup())
            EndIf
          EndIf
          
        Case 9 ; Add MP3 Folder
          path=PathRequester("Select MP3 Folder","")
          If path<>"" And ExamineDirectory(#MP3_file,path,"*.mp3")
            If ListSize(tracks())>0
              LastElement(tracks())
            EndIf
            SetCurrentDirectory(path)
            While NextDirectoryEntry(#MP3_file)
              If DirectoryEntryType(#MP3_file)=#PB_DirectoryEntry_File
                AddElement(tracks())
                tracks()\t_path=path+DirectoryEntryName(#MP3_file)
              EndIf
            Wend
            CopyList(tracks(),backup())
            FinishDirectory(#MP3_file)
            If ListSize(tracks())>-1
              Refresh_List()
              CopyList(tracks(),backup())
            EndIf
          EndIf
        
        Case 3 ; Save M3U File
          If ListSize(tracks())>0
            file=SaveFileRequester("Save M3U File","*.m3u","M3U File (*.m3u) | *.m3u",0)
            If FileSize(file)>0 : result=MessageRequester("Warning", "Overwrite Existing File?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning) : EndIf
            If result = 7 : overwrite=#False : EndIf
            If file<>"" And overwrite
              If LCase(GetExtensionPart(file))<>"m3u" : file+".m3u" : EndIf
              CreateFile(#M3U_File,file,#PB_Ascii)
              ForEach tracks()
                output=tracks()\t_path
                WriteStringN(#M3U_File,output,#PB_Ascii)
              Next
              CloseFile(#M3U_File)
            EndIf
            overwrite=#True
          EndIf
          
        Case 4 ; Save Split M3U
          If ListSize(tracks())>0
            Split_Window()
          EndIf
          
        Case 5 ; Save Track List
          If ListSize(tracks())>0
            file=SaveFileRequester("Save Text File","*.txt","Text File (*.txt) | *.txt",0)            
            If file<>""
              If Right(file,4)<>".txt" : file+".txt" : EndIf
              CreateFile(#M3U_File,file,#PB_Ascii)
              ForEach tracks()
                output=tracks()\t_artist+" - "+tracks()\t_title
                WriteStringN(#M3U_File,output,#PB_Ascii)
              Next
              CloseFile(#M3U_File)
            EndIf
          EndIf
          
        Case 6 ; Clear list
          result=MessageRequester("Clear List","Clear All Data?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)
          If result<>7 
            ClearList(tracks())
            ClearList(backup())
            ClearList(undo())
            cut=""
            If IsMovie(#MP3_Play_File) : FreeMovie(#MP3_Play_File) : mp3load=#False : EndIf
            ClearGadgetItems(#Track_List)
            mp3pos=1 : SetGadgetState(#Tool_Seek,mp3pos)
            Menu_Check()
            StatusBarText(#Main_Status,0,prog_title,#PB_StatusBar_Center)
            StatusBarText(#Main_Status,2,"0 Files",#PB_StatusBar_Center)
            StatusBarText(#Main_Status,3,"00:00:00",#PB_StatusBar_Center)
          EndIf
          
        Case 7 ; Close
          End
          
        Case 10 ; Find File
          If ListSize(tracks())>0 And GetGadgetState(#Track_List)>-1
            CopyList(tracks(),undo())
            SelectElement(tracks(),GetGadgetState(#Track_List))
            file=OpenFileRequester("Find... "+tracks()\t_path,"*.mp3","MP3 File (*.mp3) | *.mp3",0)
            If file>""
              result=MessageRequester("Is This O.K?",file,#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)
              If result<>7 : tracks()\t_path=file : EndIf
              Refresh_List()
            EndIf
          EndIf
          
        Case 11 ; Add MP3 Data
          If ListSize(tracks())>0 And GetGadgetState(#Track_List)>-1
            CopyList(tracks(),undo())
            SelectElement(tracks(),GetGadgetState(#Track_List))
            Refresh_List()
          EndIf
          
        Case 12 ; Scan All Files
          CopyList(tracks(),undo())
          If ListSize(tracks())>0
            Refresh_List()
          EndIf
          
        Case 13 ; Edit MP3 Data
          If ListSize(tracks())>0 And GetGadgetState(#Track_List)>-1
            i=0
            SelectElement(tracks(),GetGadgetState(#Track_List))
            CopyList(tracks(),undo())
            Edit_Window()
            Refresh_List()
          EndIf
          
        Case 14 ; Copy list to Clip
          If ListSize(tracks())>0
            output=""
            ForEach tracks()
              output+tracks()\t_artist+" - "+tracks()\t_title+#CRLF$
            Next
            SetClipboardText(output)
            output=""
          EndIf
          
        Case 15 ; Cut
          i=GetGadgetState(#Track_List)
          cut=""
          If i>-1
            CopyList(tracks(),undo())
            SelectElement(tracks(),i)
            cut+tracks()\t_path+#LF$
            cut+tracks()\t_artist+#LF$
            cut+tracks()\t_title+#LF$
            cut+tracks()\t_bpm+#LF$
            cut+tracks()\t_time+#LF$
            cut+tracks()\t_bitrate+#LF$
            cut+tracks()\t_vbr+#LF$
            DeleteElement(tracks(),1)
            Refresh_List()
          EndIf
          SetGadgetState(#Track_List,i-1)
          Menu_Check()
          
        Case 25 ; Delete
          i=GetGadgetState(#Track_List)          
          SelectElement(tracks(),i)
          DeleteElement(tracks(),1)
          Refresh_List()
          SetGadgetState(#Track_List,i-1)
          
        Case 19 ; Undo
          If ListSize(undo())>0
            CopyList(undo(),tracks())
            Refresh_List()
            SetActiveGadget(#Track_List)
            SetGadgetState(#Track_List,UndoLoc)
          EndIf
          
        Case 18 ; Copy
          i=GetGadgetState(#Track_List)
          cut=""
          If i>-1
            SelectElement(tracks(),i)
            cut+tracks()\t_path+#LF$
            cut+tracks()\t_artist+#LF$
            cut+tracks()\t_title+#LF$
            cut+tracks()\t_bpm+#LF$
            cut+tracks()\t_time+#LF$
            cut+tracks()\t_bitrate+#LF$
            cut+tracks()\t_vbr+#LF$
          EndIf
          SetGadgetState(#Track_List,i)
          Menu_Check()
          
        Case 16 ; Paste
          If cut>""
            CopyList(tracks(),undo())
            i=GetGadgetState(#Track_List)
            If i=-1 : i=0 : EndIf
            SelectElement(tracks(),i)
            InsertElement(tracks())
            tracks()\t_path=StringField(cut,1,#LF$)
            tracks()\t_artist=StringField(cut,2,#LF$)
            tracks()\t_title=StringField(cut,3,#LF$)
            tracks()\t_bpm=StringField(cut,4,#LF$)
            tracks()\t_time=StringField(cut,5,#LF$)
            tracks()\t_bitrate=StringField(cut,6,#LF$)
            tracks()\t_vbr=StringField(cut,7,#LF$)
            Refresh_List()
            SetGadgetState(#Track_List,i+1)
          EndIf
          
        Case 17 ; Paste Last
          If cut>""
            CopyList(tracks(),undo())
            i=GetGadgetState(#Track_List)
            AddElement(tracks())
            tracks()\t_path=StringField(cut,1,#LF$)
            tracks()\t_artist=StringField(cut,2,#LF$)
            tracks()\t_title=StringField(cut,3,#LF$)
            tracks()\t_bpm=StringField(cut,4,#LF$)
            tracks()\t_time=StringField(cut,5,#LF$)
            tracks()\t_bitrate=StringField(cut,6,#LF$)
            tracks()\t_vbr=StringField(cut,7,#LF$)
            Refresh_List()
            SetGadgetState(#Track_List,i+1)
          EndIf
          
        Case 20 ; Capitalize String
          If GetMenuItemState(#Main_Menu,20)=1
            SetMenuItemState(#Main_Menu,20,0)
            capital=#False
          Else
            SetMenuItemState(#Main_Menu,20, 1)
            capital=#True
          EndIf
          
        Case 21 ; Load with MP3 info
          If GetMenuItemState(#Main_Menu,21)=1
            SetMenuItemState(#Main_Menu,21,0)
            times=0
          Else
            SetMenuItemState(#Main_Menu,21, 1)
            times=1
          EndIf
          
        Case 22 ; refresh List
          If ListSize(tracks())>0 
            Refresh_List()
          EndIf
          
        Case 40 ; Sort By Artist
          If ListSize(tracks())>0
            CopyList(tracks(),undo())
            If sort=#True : i=#PB_Sort_Ascending : Else : i=#PB_Sort_Descending : EndIf
            SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_artist), TypeOf(Track_Data\t_artist))
            lastsort=1
            Refresh_List()
          EndIf
          
        Case 41 ; Sort By Title
          If ListSize(tracks())>0
            CopyList(tracks(),undo())
            If sort=#True : i=#PB_Sort_Ascending : Else : i=#PB_Sort_Descending : EndIf
            SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_title), TypeOf(Track_Data\t_title))
            lastsort=2
            Refresh_List()
          EndIf
          
        Case 42 ; Sort By Time
          If ListSize(tracks())>0
            CopyList(tracks(),undo())
            If sort=#True : i=#PB_Sort_Ascending : Else : i=#PB_Sort_Descending : EndIf
            SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_time), TypeOf(Track_Data\t_time))
            lastsort=3
            Refresh_List()
          EndIf
          
        Case 43 ; Sort By Path
          If ListSize(tracks())>0
            CopyList(tracks(),undo())
            If sort=#True : i=#PB_Sort_Ascending : Else : i=#PB_Sort_Descending : EndIf
            SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_path), TypeOf(Track_Data\t_path))
            lastsort=4
            Refresh_List()
          EndIf
          
        Case 46 ; Sort By BPM
          If ListSize(tracks())>0
            CopyList(tracks(),undo())
            If sort=#True : i=#PB_Sort_Ascending : Else : i=#PB_Sort_Descending : EndIf
            SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_bpm), TypeOf(Track_Data\t_bpm))
            lastsort=5
            Refresh_List()
          EndIf
          
        Case 50 ; Sort By Bitrate
          If ListSize(tracks())>0
            CopyList(tracks(),undo())
            If sort=#True : i=#PB_Sort_Ascending : Else : i=#PB_Sort_Descending : EndIf
            SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_bitrate), TypeOf(Track_Data\t_bitrate))
            lastsort=6
            Refresh_List()
          EndIf
          
        Case 44 ; Randomize List
          If ListSize(tracks())>0
            CopyList(tracks(),undo())
            For i=1 To 20
              RandomizeList(tracks())
            Next
            Refresh_List()
          EndIf   
          
        Case 45 ; Default List
          If ListSize(tracks())>0
            result=MessageRequester("Default List","Lose all changes?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)
            If result<>7
              CopyList(tracks(),undo())
              CopyList(backup(),tracks())
              Refresh_List()
            EndIf
          EndIf  
          
        Case 47 ; Remove Duplicates
          CopyList(tracks(),undo())
          Remove_Dupes()
          Refresh_List()
          
        Case 48 ; Snapshot List
          result=MessageRequester("Snapshot List","Update Default List?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)
          If result<>7
            CopyList(tracks(),backup())
          EndIf
          
        Case 49 ; Sort Ascending/Descending
          If ListSize(tracks())>0
            If sort=#True 
              SetMenuItemText(#Main_Menu,49,"Sort Descending" + Chr(9) + "Ctrl+0")
              sort=#False
              i=#PB_Sort_Descending
            Else
              SetMenuItemText(#Main_Menu,49,"Sort Ascending" + Chr(9) + "Ctrl+0")
              sort=#True
              i=#PB_Sort_Ascending
            EndIf
            Select lastsort
              Case 1 : SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_artist), TypeOf(Track_Data\t_artist))
              Case 2 : SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_title), TypeOf(Track_Data\t_title))
              Case 3 : SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_time), TypeOf(Track_Data\t_time))
              Case 4 : SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_path), TypeOf(Track_Data\t_path))
              Case 5 : SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_bpm), TypeOf(Track_Data\t_bpm))
              Case 6 : SortStructuredList(tracks(),i|#PB_Sort_NoCase , OffsetOf(Track_Data\t_bitrate), TypeOf(Track_Data\t_bitrate))
            EndSelect
            Refresh_List()                            
          EndIf
          
        Case 70 ; Play MP3
          SetActiveGadget(#Track_List)
          i=GetGadgetState(#Track_List)
          If i>-1
            SelectElement(tracks(),i)
            file=tracks()\t_path
            If mp3load : FreeMovie(#MP3_Play_File) : EndIf
            If FileSize(file)>0
              LoadMovie(#MP3_Play_File,file)
              fps=Abs(MovieInfo(#MP3_Play_File,0))/100000
              playingtrack=tracks()\t_artist+" - "+tracks()\t_title
              StatusBarText(#Main_Status,0,"Now Playing: "+playingtrack,#PB_StatusBar_Center)
              MovieAudio(#MP3_Play_File,M_Volume,0)
              PlayMovie(#MP3_Play_File,WindowID(#Main_Window))
              mp3load=#True
              Menu_Check()            
            Else
              mp3load=0
            EndIf
            
          EndIf
          
        Case 72 ; Stop MP3
          If mp3load
            StatusBarText(#Main_Status,0,GetFilePart(m3ufile),#PB_StatusBar_Center)
            StopMovie(#MP3_Play_File)
            FreeMovie(#MP3_Play_File)
            playingtrack=""
            mp3load=#False
            mp3pos=1 : SetGadgetState(#Tool_Seek,mp3pos)
            Menu_Check()
          EndIf
          
        Case 71 ; Pause/Resume MP3
          If mp3load
            If pause
              ResumeMovie(#MP3_Play_File)
              pause=#False
            Else
              PauseMovie(#MP3_Play_File)
              pause=#True
            EndIf
          EndIf
                    
        Case 76 ; Auto Search
          If ListSize(tracks())>0
            Auto_Search()
          EndIf
          
        Case 23 ; About Window
          About_Window()
          
        Case 60
          m3ufile=OpenFileRequester("Quickload PlayList","","QLD File (*.qld)|*.qld",0)
          If m3ufile<>""
            folder=GetPathPart(m3ufile)
            SetCurrentDirectory(folder)
            If OpenFile(#M3U_File,m3ufile)
              If ListSize(tracks())>-1 : ClearList(tracks()) : EndIf
              ClearGadgetItems(#Track_List)
              If LCase(GetExtensionPart(m3ufile))="qld"
                While Not Eof(#M3U_File)
                  output=ReadString(#M3U_File,ReadStringFormat(#M3U_File))         
                  AddElement(tracks())  
                  tracks()\t_path=StringField(output,1,Chr(172))
                  tracks()\t_artist=StringField(output,2,Chr(172))
                  tracks()\t_title=StringField(output,3,Chr(172))
                  tracks()\t_bpm=StringField(output,4,Chr(172))
                  tracks()\t_time=StringField(output,5,Chr(172))
                  tracks()\t_bitrate=StringField(output,6,Chr(172))
                  tracks()\t_vbr=StringField(output,7,Chr(172))
                Wend
              EndIf
              CloseFile(#M3U_File)               
              If ListSize(tracks())>-1
                c=times
                times=0
                Refresh_List()
                times=c
                CopyList(tracks(),backup())
              EndIf
            EndIf
          EndIf
          
        Case 61
          If ListSize(tracks())>0
            file=SaveFileRequester("Save Quickload","*.qld","QLD File (*.qld) | *.qld",0)
            If FileSize(file)>0 : result=MessageRequester("Warning", "Overwrite Existing File?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning) : EndIf
            If result = 7 : overwrite=#False : EndIf
            If file<>"" And overwrite
              If LCase(GetExtensionPart(file))<>"qld" : file+".qld" : EndIf
              CreateFile(#M3U_File,file,#PB_Ascii)
              ForEach tracks()
                output=tracks()\t_path+Chr(172)
                output+tracks()\t_artist+Chr(172)
                output+tracks()\t_title+Chr(172)
                output+tracks()\t_bpm+Chr(172)
                output+tracks()\t_time+Chr(172)
                output+tracks()\t_bitrate+Chr(172)
                output+tracks()\t_vbr+Chr(172)
                WriteStringN(#M3U_File,output,#PB_Ascii)
              Next
              CloseFile(#M3U_File)
            EndIf
            overwrite=#True
          EndIf
          
      EndSelect    
       
  EndSelect       
  
  If IsMovie(#MP3_Play_File) And MovieStatus(#MP3_Play_File)>0 ; Update Slider and Statusbar
    i=Abs(MovieLength(#MP3_Play_File))/fps
    j=Abs(MovieStatus(#MP3_Play_File))
    newpos=(j/i)
    If oldpos<>newpos  : StatusBarText(#Main_Status,0,"Now Playing: "+playingtrack,#PB_StatusBar_Center) : SetGadgetState(#Tool_Seek,newpos) : EndIf
    oldpos=newpos
  EndIf
  
  Delay(1)
  
Until event=#PB_Event_CloseWindow And EventWindow()=#Main_Window

DataSection
  
  OpenM3U: 
  IncludeBinary "icons\OpenM3U.png"
  
  OpenMMP: 
  IncludeBinary "icons\OpenMMP.png"
  
  SaveM3U: 
  IncludeBinary "icons\Save.png"
  
  SaveSplit: 
  IncludeBinary "icons\SaveSplit.png"
  
  Cut: 
  IncludeBinary "icons\Cut.png"
  
  Copy: 
  IncludeBinary "icons\Copy.png"
  
  Paste: 
  IncludeBinary "icons\Paste.png"
  
  Edit: 
  IncludeBinary "icons\Edit.png"
  
  Find: 
  IncludeBinary "icons\Find.png"
  
  AutoFind: 
  IncludeBinary "icons\AutoFind.png"
  
  Play: 
  IncludeBinary "icons\Play.png"
  
  Pause: 
  IncludeBinary "icons\Pause.png"
  
  Stop: 
  IncludeBinary "icons\Stop.png"
  
  Clear: 
  IncludeBinary "icons\Clear.png"
  
  Target: 
  IncludeBinary "icons\Target.png"
  
EndDataSection 
; IDE Options = PureBasic 5.50 (Windows - x64)
; CursorPosition = 1874
; FirstLine = 309
; Folding = gAAg+
; EnableThread
; EnableXP
; EnableUser
; UseIcon = headphone-icon.ico
; Executable = Playlister.exe
; Debugger = Standalone
; EnablePurifier
; IncludeVersionInfo
; VersionField0 = 1.0
; VersionField1 = 1.0
; VersionField3 = Playlister
; VersionField4 = 1.0
; VersionField5 = 1.0
; VersionField9 = Paul Vince 2016